# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class InstanceGroupManager(pulumi.CustomResource):
    auto_healing_policies: pulumi.Output[dict]
    """
    The autohealing policies for this managed instance
    group. You can specify only one value. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/creating-groups-of-managed-instances#monitoring_groups).

      * `healthCheck` (`str`) - The health check resource that signals autohealing.
      * `initialDelaySec` (`float`) - The number of seconds that the managed instance group waits before
        it applies autohealing policies to new instances or recently recreated instances. Between 0 and 3600.
    """
    base_instance_name: pulumi.Output[str]
    """
    The base instance name to use for
    instances in this group. The value must be a valid
    [RFC1035](https://www.ietf.org/rfc/rfc1035.txt) name. Supported characters
    are lowercase letters, numbers, and hyphens (-). Instances are named by
    appending a hyphen and a random four-character string to the base instance
    name.
    """
    description: pulumi.Output[str]
    """
    An optional textual description of the instance
    group manager.
    """
    fingerprint: pulumi.Output[str]
    """
    The fingerprint of the instance group manager.
    """
    instance_group: pulumi.Output[str]
    """
    The full URL of the instance group created by the manager.
    """
    name: pulumi.Output[str]
    """
    - Version name.
    """
    named_ports: pulumi.Output[list]
    """
    The named port configuration. See the section below
    for details on configuration.

      * `name` (`str`) - - Version name.
      * `port` (`float`) - The port number.
        - - -
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    self_link: pulumi.Output[str]
    """
    The URL of the created resource.
    """
    stateful_disks: pulumi.Output[list]
    """
    Disks created on the instances that will be preserved on instance delete, update, etc. Structure is documented below. For more information see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/configuring-stateful-disks-in-migs).

      * `deleteRule` (`str`) - , A value that prescribes what should happen to the stateful disk when the VM instance is deleted. The available options are `NEVER` and `ON_PERMANENT_INSTANCE_DELETION`. `NEVER` detatch the disk when the VM is deleted, but not delete the disk. `ON_PERMANENT_INSTANCE_DELETION` will delete the stateful disk when the VM is permanently deleted from the instance group. The default is `NEVER`.
      * `device_name` (`str`) - , The device name of the disk to be attached.
    """
    target_pools: pulumi.Output[list]
    """
    The full URL of all target pools to which new
    instances in the group are added. Updating the target pools attribute does
    not affect existing instances.
    """
    target_size: pulumi.Output[float]
    """
    - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
    """
    update_policy: pulumi.Output[dict]
    """
    The update policy for this managed instance group. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/updating-managed-instance-groups) and [API](https://cloud.google.com/compute/docs/reference/rest/beta/instanceGroupManagers/patch)

      * `maxSurgeFixed` (`float`) - , The maximum number of instances that can be created above the specified targetSize during the update process. Conflicts with `max_surge_percent`. If neither is set, defaults to 1
      * `maxSurgePercent` (`float`) - , The maximum number of instances(calculated as percentage) that can be created above the specified targetSize during the update process. Conflicts with `max_surge_fixed`.
      * `maxUnavailableFixed` (`float`) - , The maximum number of instances that can be unavailable during the update process. Conflicts with `max_unavailable_percent`. If neither is set, defaults to 1
      * `maxUnavailablePercent` (`float`) - , The maximum number of instances(calculated as percentage) that can be unavailable during the update process. Conflicts with `max_unavailable_fixed`.
      * `minReadySec` (`float`) - , Minimum number of seconds to wait for after a newly created instance becomes available. This value must be from range [0, 3600]
        - - -
      * `minimal_action` (`str`) - - Minimal action to be taken on an instance. You can specify either `RESTART` to restart existing instances or `REPLACE` to delete and create new instances from the target template. If you specify a `RESTART`, the Updater will attempt to perform that action only. However, if the Updater determines that the minimal action you specify is not enough to perform the update, it might perform a more disruptive action.
      * `type` (`str`) - - The type of update process. You can specify either `PROACTIVE` so that the instance group manager proactively executes actions in order to bring instances to their target versions or `OPPORTUNISTIC` so that no action is proactively executed but the update will be performed as part of other actions (for example, resizes or recreateInstances calls).
    """
    versions: pulumi.Output[list]
    """
    Application versions managed by this instance group. Each
    version deals with a specific instance template, allowing canary release scenarios.
    Structure is documented below.

      * `instanceTemplate` (`str`) - - The full URL to an instance template from which all new instances of this version will be created.
      * `name` (`str`) - - Version name.
      * `target_size` (`dict`) - - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
        * `fixed` (`float`) - , The number of instances which are managed for this version. Conflicts with `percent`.
        * `percent` (`float`) - , The number of instances (calculated as percentage) which are managed for this version. Conflicts with `fixed`.
          Note that when using `percent`, rounding will be in favor of explicitly set `target_size` values; a managed instance group with 2 instances and 2 `version`s,
          one of which has a `target_size.percent` of `60` will create 2 instances of that `version`.
    """
    wait_for_instances: pulumi.Output[bool]
    """
    Whether to wait for all instances to be created/updated before
    returning. Note that if this is set to true and the operation does not succeed, this provider will
    continue trying until it times out.
    """
    zone: pulumi.Output[str]
    """
    The zone that instances in this group should be created
    in.
    """
    def __init__(__self__, resource_name, opts=None, auto_healing_policies=None, base_instance_name=None, description=None, name=None, named_ports=None, project=None, stateful_disks=None, target_pools=None, target_size=None, update_policy=None, versions=None, wait_for_instances=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        The Google Compute Engine Instance Group Manager API creates and manages pools
        of homogeneous Compute Engine virtual machine instances from a common instance
        template. For more information, see [the official documentation](https://cloud.google.com/compute/docs/instance-groups/manager)
        and [API](https://cloud.google.com/compute/docs/reference/latest/instanceGroupManagers)

        > **Note:** Use [compute.RegionInstanceGroupManager](https://www.terraform.io/docs/providers/google/r/compute_region_instance_group_manager.html) to create a regional (multi-zone) instance group manager.

        ## Example Usage

        ### With Top Level Instance Template (`Google` Provider)

        ```python
        import pulumi
        import pulumi_gcp as gcp

        autohealing = gcp.compute.HealthCheck("autohealing",
            check_interval_sec=5,
            timeout_sec=5,
            healthy_threshold=2,
            unhealthy_threshold=10,
            http_health_check={
                "request_path": "/healthz",
                "port": "8080",
            })
        appserver = gcp.compute.InstanceGroupManager("appserver",
            base_instance_name="app",
            zone="us-central1-a",
            version=[{
                "instanceTemplate": google_compute_instance_template["appserver"]["id"],
            }],
            target_pools=[google_compute_target_pool["appserver"]["id"]],
            target_size=2,
            named_port=[{
                "name": "customHTTP",
                "port": 8888,
            }],
            auto_healing_policies={
                "healthCheck": autohealing.id,
                "initialDelaySec": 300,
            })
        ```

        ### With Multiple Versions (`Google-Beta` Provider)

        ```python
        import pulumi
        import pulumi_gcp as gcp

        appserver = gcp.compute.InstanceGroupManager("appserver",
            base_instance_name="app",
            zone="us-central1-a",
            target_size=5,
            version=[
                {
                    "name": "appserver",
                    "instanceTemplate": google_compute_instance_template["appserver"]["id"],
                },
                {
                    "name": "appserver-canary",
                    "instanceTemplate": google_compute_instance_template["appserver-canary"]["id"],
                    "target_size": {
                        "fixed": 1,
                    },
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] auto_healing_policies: The autohealing policies for this managed instance
               group. You can specify only one value. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/creating-groups-of-managed-instances#monitoring_groups).
        :param pulumi.Input[str] base_instance_name: The base instance name to use for
               instances in this group. The value must be a valid
               [RFC1035](https://www.ietf.org/rfc/rfc1035.txt) name. Supported characters
               are lowercase letters, numbers, and hyphens (-). Instances are named by
               appending a hyphen and a random four-character string to the base instance
               name.
        :param pulumi.Input[str] description: An optional textual description of the instance
               group manager.
        :param pulumi.Input[str] name: - Version name.
        :param pulumi.Input[list] named_ports: The named port configuration. See the section below
               for details on configuration.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[list] stateful_disks: Disks created on the instances that will be preserved on instance delete, update, etc. Structure is documented below. For more information see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/configuring-stateful-disks-in-migs).
        :param pulumi.Input[list] target_pools: The full URL of all target pools to which new
               instances in the group are added. Updating the target pools attribute does
               not affect existing instances.
        :param pulumi.Input[float] target_size: - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
        :param pulumi.Input[dict] update_policy: The update policy for this managed instance group. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/updating-managed-instance-groups) and [API](https://cloud.google.com/compute/docs/reference/rest/beta/instanceGroupManagers/patch)
        :param pulumi.Input[list] versions: Application versions managed by this instance group. Each
               version deals with a specific instance template, allowing canary release scenarios.
               Structure is documented below.
        :param pulumi.Input[bool] wait_for_instances: Whether to wait for all instances to be created/updated before
               returning. Note that if this is set to true and the operation does not succeed, this provider will
               continue trying until it times out.
        :param pulumi.Input[str] zone: The zone that instances in this group should be created
               in.

        The **auto_healing_policies** object supports the following:

          * `healthCheck` (`pulumi.Input[str]`) - The health check resource that signals autohealing.
          * `initialDelaySec` (`pulumi.Input[float]`) - The number of seconds that the managed instance group waits before
            it applies autohealing policies to new instances or recently recreated instances. Between 0 and 3600.

        The **named_ports** object supports the following:

          * `name` (`pulumi.Input[str]`) - - Version name.
          * `port` (`pulumi.Input[float]`) - The port number.
            - - -

        The **stateful_disks** object supports the following:

          * `deleteRule` (`pulumi.Input[str]`) - , A value that prescribes what should happen to the stateful disk when the VM instance is deleted. The available options are `NEVER` and `ON_PERMANENT_INSTANCE_DELETION`. `NEVER` detatch the disk when the VM is deleted, but not delete the disk. `ON_PERMANENT_INSTANCE_DELETION` will delete the stateful disk when the VM is permanently deleted from the instance group. The default is `NEVER`.
          * `device_name` (`pulumi.Input[str]`) - , The device name of the disk to be attached.

        The **update_policy** object supports the following:

          * `maxSurgeFixed` (`pulumi.Input[float]`) - , The maximum number of instances that can be created above the specified targetSize during the update process. Conflicts with `max_surge_percent`. If neither is set, defaults to 1
          * `maxSurgePercent` (`pulumi.Input[float]`) - , The maximum number of instances(calculated as percentage) that can be created above the specified targetSize during the update process. Conflicts with `max_surge_fixed`.
          * `maxUnavailableFixed` (`pulumi.Input[float]`) - , The maximum number of instances that can be unavailable during the update process. Conflicts with `max_unavailable_percent`. If neither is set, defaults to 1
          * `maxUnavailablePercent` (`pulumi.Input[float]`) - , The maximum number of instances(calculated as percentage) that can be unavailable during the update process. Conflicts with `max_unavailable_fixed`.
          * `minReadySec` (`pulumi.Input[float]`) - , Minimum number of seconds to wait for after a newly created instance becomes available. This value must be from range [0, 3600]
            - - -
          * `minimal_action` (`pulumi.Input[str]`) - - Minimal action to be taken on an instance. You can specify either `RESTART` to restart existing instances or `REPLACE` to delete and create new instances from the target template. If you specify a `RESTART`, the Updater will attempt to perform that action only. However, if the Updater determines that the minimal action you specify is not enough to perform the update, it might perform a more disruptive action.
          * `type` (`pulumi.Input[str]`) - - The type of update process. You can specify either `PROACTIVE` so that the instance group manager proactively executes actions in order to bring instances to their target versions or `OPPORTUNISTIC` so that no action is proactively executed but the update will be performed as part of other actions (for example, resizes or recreateInstances calls).

        The **versions** object supports the following:

          * `instanceTemplate` (`pulumi.Input[str]`) - - The full URL to an instance template from which all new instances of this version will be created.
          * `name` (`pulumi.Input[str]`) - - Version name.
          * `target_size` (`pulumi.Input[dict]`) - - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
            * `fixed` (`pulumi.Input[float]`) - , The number of instances which are managed for this version. Conflicts with `percent`.
            * `percent` (`pulumi.Input[float]`) - , The number of instances (calculated as percentage) which are managed for this version. Conflicts with `fixed`.
              Note that when using `percent`, rounding will be in favor of explicitly set `target_size` values; a managed instance group with 2 instances and 2 `version`s,
              one of which has a `target_size.percent` of `60` will create 2 instances of that `version`.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['auto_healing_policies'] = auto_healing_policies
            if base_instance_name is None:
                raise TypeError("Missing required property 'base_instance_name'")
            __props__['base_instance_name'] = base_instance_name
            __props__['description'] = description
            __props__['name'] = name
            __props__['named_ports'] = named_ports
            __props__['project'] = project
            __props__['stateful_disks'] = stateful_disks
            __props__['target_pools'] = target_pools
            __props__['target_size'] = target_size
            __props__['update_policy'] = update_policy
            if versions is None:
                raise TypeError("Missing required property 'versions'")
            __props__['versions'] = versions
            __props__['wait_for_instances'] = wait_for_instances
            __props__['zone'] = zone
            __props__['fingerprint'] = None
            __props__['instance_group'] = None
            __props__['self_link'] = None
        super(InstanceGroupManager, __self__).__init__(
            'gcp:compute/instanceGroupManager:InstanceGroupManager',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, auto_healing_policies=None, base_instance_name=None, description=None, fingerprint=None, instance_group=None, name=None, named_ports=None, project=None, self_link=None, stateful_disks=None, target_pools=None, target_size=None, update_policy=None, versions=None, wait_for_instances=None, zone=None):
        """
        Get an existing InstanceGroupManager resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] auto_healing_policies: The autohealing policies for this managed instance
               group. You can specify only one value. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/creating-groups-of-managed-instances#monitoring_groups).
        :param pulumi.Input[str] base_instance_name: The base instance name to use for
               instances in this group. The value must be a valid
               [RFC1035](https://www.ietf.org/rfc/rfc1035.txt) name. Supported characters
               are lowercase letters, numbers, and hyphens (-). Instances are named by
               appending a hyphen and a random four-character string to the base instance
               name.
        :param pulumi.Input[str] description: An optional textual description of the instance
               group manager.
        :param pulumi.Input[str] fingerprint: The fingerprint of the instance group manager.
        :param pulumi.Input[str] instance_group: The full URL of the instance group created by the manager.
        :param pulumi.Input[str] name: - Version name.
        :param pulumi.Input[list] named_ports: The named port configuration. See the section below
               for details on configuration.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[str] self_link: The URL of the created resource.
        :param pulumi.Input[list] stateful_disks: Disks created on the instances that will be preserved on instance delete, update, etc. Structure is documented below. For more information see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/configuring-stateful-disks-in-migs).
        :param pulumi.Input[list] target_pools: The full URL of all target pools to which new
               instances in the group are added. Updating the target pools attribute does
               not affect existing instances.
        :param pulumi.Input[float] target_size: - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
        :param pulumi.Input[dict] update_policy: The update policy for this managed instance group. Structure is documented below. For more information, see the [official documentation](https://cloud.google.com/compute/docs/instance-groups/updating-managed-instance-groups) and [API](https://cloud.google.com/compute/docs/reference/rest/beta/instanceGroupManagers/patch)
        :param pulumi.Input[list] versions: Application versions managed by this instance group. Each
               version deals with a specific instance template, allowing canary release scenarios.
               Structure is documented below.
        :param pulumi.Input[bool] wait_for_instances: Whether to wait for all instances to be created/updated before
               returning. Note that if this is set to true and the operation does not succeed, this provider will
               continue trying until it times out.
        :param pulumi.Input[str] zone: The zone that instances in this group should be created
               in.

        The **auto_healing_policies** object supports the following:

          * `healthCheck` (`pulumi.Input[str]`) - The health check resource that signals autohealing.
          * `initialDelaySec` (`pulumi.Input[float]`) - The number of seconds that the managed instance group waits before
            it applies autohealing policies to new instances or recently recreated instances. Between 0 and 3600.

        The **named_ports** object supports the following:

          * `name` (`pulumi.Input[str]`) - - Version name.
          * `port` (`pulumi.Input[float]`) - The port number.
            - - -

        The **stateful_disks** object supports the following:

          * `deleteRule` (`pulumi.Input[str]`) - , A value that prescribes what should happen to the stateful disk when the VM instance is deleted. The available options are `NEVER` and `ON_PERMANENT_INSTANCE_DELETION`. `NEVER` detatch the disk when the VM is deleted, but not delete the disk. `ON_PERMANENT_INSTANCE_DELETION` will delete the stateful disk when the VM is permanently deleted from the instance group. The default is `NEVER`.
          * `device_name` (`pulumi.Input[str]`) - , The device name of the disk to be attached.

        The **update_policy** object supports the following:

          * `maxSurgeFixed` (`pulumi.Input[float]`) - , The maximum number of instances that can be created above the specified targetSize during the update process. Conflicts with `max_surge_percent`. If neither is set, defaults to 1
          * `maxSurgePercent` (`pulumi.Input[float]`) - , The maximum number of instances(calculated as percentage) that can be created above the specified targetSize during the update process. Conflicts with `max_surge_fixed`.
          * `maxUnavailableFixed` (`pulumi.Input[float]`) - , The maximum number of instances that can be unavailable during the update process. Conflicts with `max_unavailable_percent`. If neither is set, defaults to 1
          * `maxUnavailablePercent` (`pulumi.Input[float]`) - , The maximum number of instances(calculated as percentage) that can be unavailable during the update process. Conflicts with `max_unavailable_fixed`.
          * `minReadySec` (`pulumi.Input[float]`) - , Minimum number of seconds to wait for after a newly created instance becomes available. This value must be from range [0, 3600]
            - - -
          * `minimal_action` (`pulumi.Input[str]`) - - Minimal action to be taken on an instance. You can specify either `RESTART` to restart existing instances or `REPLACE` to delete and create new instances from the target template. If you specify a `RESTART`, the Updater will attempt to perform that action only. However, if the Updater determines that the minimal action you specify is not enough to perform the update, it might perform a more disruptive action.
          * `type` (`pulumi.Input[str]`) - - The type of update process. You can specify either `PROACTIVE` so that the instance group manager proactively executes actions in order to bring instances to their target versions or `OPPORTUNISTIC` so that no action is proactively executed but the update will be performed as part of other actions (for example, resizes or recreateInstances calls).

        The **versions** object supports the following:

          * `instanceTemplate` (`pulumi.Input[str]`) - - The full URL to an instance template from which all new instances of this version will be created.
          * `name` (`pulumi.Input[str]`) - - Version name.
          * `target_size` (`pulumi.Input[dict]`) - - The number of instances calculated as a fixed number or a percentage depending on the settings. Structure is documented below.
            * `fixed` (`pulumi.Input[float]`) - , The number of instances which are managed for this version. Conflicts with `percent`.
            * `percent` (`pulumi.Input[float]`) - , The number of instances (calculated as percentage) which are managed for this version. Conflicts with `fixed`.
              Note that when using `percent`, rounding will be in favor of explicitly set `target_size` values; a managed instance group with 2 instances and 2 `version`s,
              one of which has a `target_size.percent` of `60` will create 2 instances of that `version`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["auto_healing_policies"] = auto_healing_policies
        __props__["base_instance_name"] = base_instance_name
        __props__["description"] = description
        __props__["fingerprint"] = fingerprint
        __props__["instance_group"] = instance_group
        __props__["name"] = name
        __props__["named_ports"] = named_ports
        __props__["project"] = project
        __props__["self_link"] = self_link
        __props__["stateful_disks"] = stateful_disks
        __props__["target_pools"] = target_pools
        __props__["target_size"] = target_size
        __props__["update_policy"] = update_policy
        __props__["versions"] = versions
        __props__["wait_for_instances"] = wait_for_instances
        __props__["zone"] = zone
        return InstanceGroupManager(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class FlexibleAppVersion(pulumi.CustomResource):
    api_config: pulumi.Output[dict]
    """
    Serving configuration for Google Cloud Endpoints.  Structure is documented below.

      * `authFailAction` (`str`) - Action to take when users access resources that require authentication.
      * `login` (`str`) - Level of login required to access this resource.
      * `script` (`str`) - Path to the script from the application root directory.
      * `securityLevel` (`str`) - Security (HTTPS) enforcement for this URL.
      * `url` (`str`) - URL to serve the endpoint at.
    """
    automatic_scaling: pulumi.Output[dict]
    """
    Automatic scaling is based on request rate, response latencies, and other application metrics.  Structure is documented below.

      * `coolDownPeriod` (`str`) - The time period that the Autoscaler should wait before it starts collecting information from a new instance.
        This prevents the autoscaler from collecting information when the instance is initializing,
        during which the collected usage would not be reliable. Default: 120s
      * `cpuUtilization` (`dict`) - Target scaling by CPU usage.  Structure is documented below.
        * `aggregationWindowLength` (`str`) - Period of time over which CPU utilization is calculated.
        * `targetUtilization` (`float`) - Target CPU utilization ratio to maintain when scaling. Must be between 0 and 1.

      * `diskUtilization` (`dict`) - Target scaling by disk usage.  Structure is documented below.
        * `targetReadBytesPerSecond` (`float`) - Target bytes read per second.
        * `targetReadOpsPerSecond` (`float`) - Target ops read per seconds.
        * `targetWriteBytesPerSecond` (`float`) - Target bytes written per second.
        * `targetWriteOpsPerSecond` (`float`) - Target ops written per second.

      * `maxConcurrentRequests` (`float`) - Number of concurrent requests an automatic scaling instance can accept before the scheduler spawns a new instance.
        Defaults to a runtime-specific value.
      * `maxIdleInstances` (`float`) - Maximum number of idle instances that should be maintained for this version.
      * `maxPendingLatency` (`str`) - Maximum amount of time that a request should wait in the pending queue before starting a new instance to handle it.
      * `maxTotalInstances` (`float`) - Maximum number of instances that should be started to handle requests for this version. Default: 20
      * `minIdleInstances` (`float`) - Minimum number of idle instances that should be maintained for this version. Only applicable for the default version of a service.
      * `minPendingLatency` (`str`) - Minimum amount of time a request should wait in the pending queue before starting a new instance to handle it.
      * `minTotalInstances` (`float`) - Minimum number of running instances that should be maintained for this version. Default: 2
      * `networkUtilization` (`dict`) - Target scaling by network usage.  Structure is documented below.
        * `targetReceivedBytesPerSecond` (`float`) - Target bytes received per second.
        * `targetReceivedPacketsPerSecond` (`float`) - Target packets received per second.
        * `targetSentBytesPerSecond` (`float`) - Target bytes sent per second.
        * `targetSentPacketsPerSecond` (`float`) - Target packets sent per second.

      * `requestUtilization` (`dict`) - Target scaling by request utilization.  Structure is documented below.
        * `targetConcurrentRequests` (`float`) - Target number of concurrent requests.
        * `targetRequestCountPerSecond` (`str`) - Target requests per second.
    """
    beta_settings: pulumi.Output[dict]
    """
    Metadata settings that are supplied to this version to enable beta runtime features.
    """
    default_expiration: pulumi.Output[str]
    """
    Duration that static files should be cached by web proxies and browsers.
    Only applicable if the corresponding StaticFilesHandler does not specify its own expiration time.
    """
    delete_service_on_destroy: pulumi.Output[bool]
    """
    If set to `true`, the service will be deleted if it is the last version.
    """
    deployment: pulumi.Output[dict]
    """
    Code and application artifacts that make up this version.  Structure is documented below.

      * `cloudBuildOptions` (`dict`) - Options for the build operations performed as a part of the version deployment. Only applicable when creating a version using source code directly.  Structure is documented below.
        * `appYamlPath` (`str`) - Path to the yaml file used in deployment, used to determine runtime configuration details.
        * `cloudBuildTimeout` (`str`) - The Cloud Build timeout used as part of any dependent builds performed by version creation. Defaults to 10 minutes.
          A duration in seconds with up to nine fractional digits, terminated by 's'. Example: "3.5s".

      * `container` (`dict`) - The Docker image for the container that runs the version.  Structure is documented below.
        * `image` (`str`) - URI to the hosted container image in Google Container Registry. The URI must be fully qualified and include a tag or digest.
          Examples: "gcr.io/my-project/image:tag" or "gcr.io/my-project/image@digest"

      * `files` (`list`) - Manifest of the files stored in Google Cloud Storage that are included as part of this version.
        All files must be readable using the credentials supplied with this call.  Structure is documented below.
        * `name` (`str`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
        * `sha1Sum` (`str`) - SHA1 checksum of the file
        * `sourceUrl` (`str`) - Source URL

      * `zip` (`dict`) - Zip File  Structure is documented below.
        * `filesCount` (`float`) - files count
        * `sourceUrl` (`str`) - Source URL
    """
    endpoints_api_service: pulumi.Output[dict]
    """
    Code and application artifacts that make up this version.  Structure is documented below.

      * `config_id` (`str`) - Endpoints service configuration ID as specified by the Service Management API. For example "2016-09-19r1".
        By default, the rollout strategy for Endpoints is "FIXED". This means that Endpoints starts up with a particular configuration ID.
        When a new configuration is rolled out, Endpoints must be given the new configuration ID. The configId field is used to give the configuration ID
        and is required in this case.
        Endpoints also has a rollout strategy called "MANAGED". When using this, Endpoints fetches the latest configuration and does not need
        the configuration ID. In this case, configId must be omitted.
      * `disableTraceSampling` (`bool`) - Enable or disable trace sampling. By default, this is set to false for enabled.
      * `name` (`str`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
      * `rolloutStrategy` (`str`) - Endpoints rollout strategy. If FIXED, configId must be specified. If MANAGED, configId must be omitted.
    """
    entrypoint: pulumi.Output[dict]
    """
    The entrypoint for the application.  Structure is documented below.

      * `shell` (`str`) - The format should be a shell command that can be fed to bash -c.
    """
    env_variables: pulumi.Output[dict]
    """
    Environment variables available to the application.  As these are not returned in the API request, the provider will not detect any changes made outside of the config.
    """
    handlers: pulumi.Output[list]
    """
    An ordered list of URL-matching patterns that should be applied to incoming requests.
    The first matching URL handles the request and other request handlers are not attempted.  Structure is documented below.

      * `authFailAction` (`str`) - Action to take when users access resources that require authentication.
      * `login` (`str`) - Level of login required to access this resource.
      * `redirectHttpResponseCode` (`str`) - 30x code to use when performing redirects for the secure field.
      * `script` (`dict`) - Path to the script from the application root directory.
        * `scriptPath` (`str`) - Path to the script from the application root directory.

      * `securityLevel` (`str`) - Security (HTTPS) enforcement for this URL.
      * `staticFiles` (`dict`) - Files served directly to the user for a given URL, such as images, CSS stylesheets, or JavaScript source files.
        Static file handlers describe which files in the application directory are static files, and which URLs serve them.  Structure is documented below.
        * `applicationReadable` (`bool`) - Whether files should also be uploaded as code data. By default, files declared in static file handlers are
          uploaded as static data and are only served to end users; they cannot be read by the application. If enabled,
          uploads are charged against both your code and static data storage resource quotas.
        * `expiration` (`str`) - Time a static file served by this handler should be cached by web proxies and browsers.
          A duration in seconds with up to nine fractional digits, terminated by 's'. Example "3.5s".
          Default is '0s'
        * `httpHeaders` (`dict`) - HTTP headers to use for all responses from these URLs.
          An object containing a list of "key:value" value pairs.".
        * `mimeType` (`str`) - MIME type used to serve all files served by this handler.
          Defaults to file-specific MIME types, which are derived from each file's filename extension.
        * `path` (`str`) - Path to the static files matched by the URL pattern, from the application root directory.
          The path can refer to text matched in groupings in the URL pattern.
        * `requireMatchingFile` (`bool`) - Whether this handler should match the request if the file referenced by the handler does not exist.
        * `uploadPathRegex` (`str`) - Regular expression that matches the file paths for all files that should be referenced by this handler.

      * `urlRegex` (`str`) - URL prefix. Uses regular expression syntax, which means regexp special characters must be escaped, but should not contain groupings.
        All URLs that begin with this prefix are handled by this handler, using the portion of the URL after the prefix as part of the file path.
    """
    inbound_services: pulumi.Output[list]
    """
    Before an application can receive email or XMPP messages, the application must be configured to enable the service.
    """
    instance_class: pulumi.Output[str]
    """
    Instance class that is used to run this version. Valid values are
    AutomaticScaling: F1, F2, F4, F4_1G
    ManualScaling: B1, B2, B4, B8, B4_1G
    Defaults to F1 for AutomaticScaling and B1 for ManualScaling.
    """
    liveness_check: pulumi.Output[dict]
    """
    Health checking configuration for VM instances. Unhealthy instances are killed and replaced with new instances.  Structure is documented below.

      * `checkInterval` (`str`) - Interval between health checks.
      * `failureThreshold` (`float`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
      * `host` (`str`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
      * `initialDelay` (`str`) - The initial delay before starting to execute the checks. Default: "300s"
      * `path` (`str`) - Path to the static files matched by the URL pattern, from the application root directory.
        The path can refer to text matched in groupings in the URL pattern.
      * `successThreshold` (`float`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
      * `timeout` (`str`) - Time before the check is considered failed. Default: "4s"
    """
    manual_scaling: pulumi.Output[dict]
    """
    A service with manual scaling runs continuously, allowing you to perform complex initialization and rely on the state of its memory over time.  Structure is documented below.

      * `instances` (`float`) - Number of instances to assign to the service at the start.
        **Note:** When managing the number of instances at runtime through the App Engine Admin API or the (now deprecated) Python 2
        Modules API set_num_instances() you must use `lifecycle.ignore_changes = ["manual_scaling"[0].instances]` to prevent drift detection.
    """
    name: pulumi.Output[str]
    """
    Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
    """
    network: pulumi.Output[dict]
    """
    Extra network settings  Structure is documented below.

      * `forwardedPorts` (`list`) - List of ports, or port pairs, to forward from the virtual machine to the application container.
      * `instanceTag` (`str`) - Tag to apply to the instance during creation.
      * `name` (`str`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
      * `session_affinity` (`bool`) - Enable session affinity.
      * `subnetwork` (`str`) - Google Cloud Platform sub-network where the virtual machines are created. Specify the short name, not the resource path.
        If the network that the instance is being created in is a Legacy network, then the IP address is allocated from the IPv4Range.
        If the network that the instance is being created in is an auto Subnet Mode Network, then only network name should be specified (not the subnetworkName) and the IP address is created from the IPCidrRange of the subnetwork that exists in that zone for that network.
        If the network that the instance is being created in is a custom Subnet Mode Network, then the subnetworkName must be specified and the IP address is created from the IPCidrRange of the subnetwork.
        If specified, the subnetwork must exist in the same region as the App Engine flexible environment application.
    """
    nobuild_files_regex: pulumi.Output[str]
    """
    Files that match this pattern will not be built into this version. Only applicable for Go runtimes.
    """
    noop_on_destroy: pulumi.Output[bool]
    """
    If set to `true`, the application version will not be deleted.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    readiness_check: pulumi.Output[dict]
    """
    Configures readiness health checking for instances. Unhealthy instances are not put into the backend traffic rotation.  Structure is documented below.

      * `appStartTimeout` (`str`) - A maximum time limit on application initialization, measured from moment the application successfully
        replies to a healthcheck until it is ready to serve traffic. Default: "300s"
      * `checkInterval` (`str`) - Interval between health checks.
      * `failureThreshold` (`float`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
      * `host` (`str`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
      * `path` (`str`) - Path to the static files matched by the URL pattern, from the application root directory.
        The path can refer to text matched in groupings in the URL pattern.
      * `successThreshold` (`float`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
      * `timeout` (`str`) - Time before the check is considered failed. Default: "4s"
    """
    resources: pulumi.Output[dict]
    """
    Machine resources for a version.  Structure is documented below.

      * `cpu` (`float`) - Number of CPU cores needed.
      * `diskGb` (`float`) - Disk size (GB) needed.
      * `memoryGb` (`float`) - Memory (GB) needed.
      * `volumes` (`list`) - List of ports, or port pairs, to forward from the virtual machine to the application container.  Structure is documented below.
        * `name` (`str`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
        * `sizeGb` (`float`) - Volume size in gigabytes.
        * `volumeType` (`str`) - Underlying volume type, e.g. 'tmpfs'.
    """
    runtime: pulumi.Output[str]
    """
    Desired runtime. Example python27.
    """
    runtime_api_version: pulumi.Output[str]
    """
    The version of the API in the given runtime environment.
    Please see the app.yaml reference for valid values at https://cloud.google.com/appengine/docs/standard//config/appref
    """
    runtime_channel: pulumi.Output[str]
    """
    The channel of the runtime to use. Only available for some runtimes.
    """
    runtime_main_executable_path: pulumi.Output[str]
    """
    The path or name of the app's main executable.
    """
    service: pulumi.Output[str]
    """
    AppEngine service resource
    """
    serving_status: pulumi.Output[str]
    """
    Current serving status of this version. Only the versions with a SERVING status create instances and can be billed.
    """
    version_id: pulumi.Output[str]
    """
    Relative name of the version within the service. For example, `v1`. Version names can contain only lowercase letters, numbers, or hyphens.
    Reserved names,"default", "latest", and any name with the prefix "ah-".
    """
    vpc_access_connector: pulumi.Output[dict]
    """
    Enables VPC connectivity for standard apps.  Structure is documented below.

      * `name` (`str`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
    """
    def __init__(__self__, resource_name, opts=None, api_config=None, automatic_scaling=None, beta_settings=None, default_expiration=None, delete_service_on_destroy=None, deployment=None, endpoints_api_service=None, entrypoint=None, env_variables=None, handlers=None, inbound_services=None, instance_class=None, liveness_check=None, manual_scaling=None, network=None, nobuild_files_regex=None, noop_on_destroy=None, project=None, readiness_check=None, resources=None, runtime=None, runtime_api_version=None, runtime_channel=None, runtime_main_executable_path=None, service=None, serving_status=None, version_id=None, vpc_access_connector=None, __props__=None, __name__=None, __opts__=None):
        """
        Flexible App Version resource to create a new version of flexible GAE Application. Based on Google Compute Engine,
        the App Engine flexible environment automatically scales your app up and down while also balancing the load.
        Learn about the differences between the standard environment and the flexible environment
        at https://cloud.google.com/appengine/docs/the-appengine-environments.

        > **Note:** The App Engine flexible environment service account uses the member ID `service-[YOUR_PROJECT_NUMBER]@gae-api-prod.google.com.iam.gserviceaccount.com`
        It should have the App Engine Flexible Environment Service Agent role, which will be applied when the `appengineflex.googleapis.com` service is enabled.


        To get more information about FlexibleAppVersion, see:

        * [API documentation](https://cloud.google.com/appengine/docs/admin-api/reference/rest/v1/apps.services.versions)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/appengine/docs/flexible)

        ## Example Usage

        ### App Engine Flexible App Version

        ```python
        import pulumi
        import pulumi_gcp as gcp

        my_project = gcp.organizations.Project("myProject",
            project_id="appeng-flex",
            org_id="123456789",
            billing_account="000000-0000000-0000000-000000")
        app = gcp.appengine.Application("app",
            project=my_project.project_id,
            location_id="us-central")
        service = gcp.projects.Service("service",
            project=my_project.project_id,
            service="appengineflex.googleapis.com",
            disable_dependent_services=False)
        gae_api = gcp.projects.IAMMember("gaeApi",
            project=service.project,
            role="roles/compute.networkUser",
            member=my_project.number.apply(lambda number: f"serviceAccount:service-{number}@gae-api-prod.google.com.iam.gserviceaccount.com"))
        bucket = gcp.storage.Bucket("bucket", project=my_project.project_id)
        object = gcp.storage.BucketObject("object",
            bucket=bucket.name,
            source=pulumi.FileAsset("./test-fixtures/appengine/hello-world.zip"))
        myapp_v1 = gcp.appengine.FlexibleAppVersion("myappV1",
            version_id="v1",
            project=gae_api.project,
            service="default",
            runtime="nodejs",
            entrypoint={
                "shell": "node ./app.js",
            },
            deployment={
                "zip": {
                    "sourceUrl": pulumi.Output.all(bucket.name, object.name).apply(lambda bucketName, objectName: f"https://storage.googleapis.com/{bucket_name}/{object_name}"),
                },
            },
            liveness_check={
                "path": "/",
            },
            readiness_check={
                "path": "/",
            },
            env_variables={
                "port": "8080",
            },
            handlers=[{
                "urlRegex": ".*\\/my-path\\/*",
                "securityLevel": "SECURE_ALWAYS",
                "login": "LOGIN_REQUIRED",
                "authFailAction": "AUTH_FAIL_ACTION_REDIRECT",
                "static_files": {
                    "path": "my-other-path",
                    "uploadPathRegex": ".*\\/my-path\\/*",
                },
            }],
            automatic_scaling={
                "coolDownPeriod": "120s",
                "cpu_utilization": {
                    "targetUtilization": 0.5,
                },
            },
            noop_on_destroy=True)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] api_config: Serving configuration for Google Cloud Endpoints.  Structure is documented below.
        :param pulumi.Input[dict] automatic_scaling: Automatic scaling is based on request rate, response latencies, and other application metrics.  Structure is documented below.
        :param pulumi.Input[dict] beta_settings: Metadata settings that are supplied to this version to enable beta runtime features.
        :param pulumi.Input[str] default_expiration: Duration that static files should be cached by web proxies and browsers.
               Only applicable if the corresponding StaticFilesHandler does not specify its own expiration time.
        :param pulumi.Input[bool] delete_service_on_destroy: If set to `true`, the service will be deleted if it is the last version.
        :param pulumi.Input[dict] deployment: Code and application artifacts that make up this version.  Structure is documented below.
        :param pulumi.Input[dict] endpoints_api_service: Code and application artifacts that make up this version.  Structure is documented below.
        :param pulumi.Input[dict] entrypoint: The entrypoint for the application.  Structure is documented below.
        :param pulumi.Input[dict] env_variables: Environment variables available to the application.  As these are not returned in the API request, the provider will not detect any changes made outside of the config.
        :param pulumi.Input[list] handlers: An ordered list of URL-matching patterns that should be applied to incoming requests.
               The first matching URL handles the request and other request handlers are not attempted.  Structure is documented below.
        :param pulumi.Input[list] inbound_services: Before an application can receive email or XMPP messages, the application must be configured to enable the service.
        :param pulumi.Input[str] instance_class: Instance class that is used to run this version. Valid values are
               AutomaticScaling: F1, F2, F4, F4_1G
               ManualScaling: B1, B2, B4, B8, B4_1G
               Defaults to F1 for AutomaticScaling and B1 for ManualScaling.
        :param pulumi.Input[dict] liveness_check: Health checking configuration for VM instances. Unhealthy instances are killed and replaced with new instances.  Structure is documented below.
        :param pulumi.Input[dict] manual_scaling: A service with manual scaling runs continuously, allowing you to perform complex initialization and rely on the state of its memory over time.  Structure is documented below.
        :param pulumi.Input[dict] network: Extra network settings  Structure is documented below.
        :param pulumi.Input[str] nobuild_files_regex: Files that match this pattern will not be built into this version. Only applicable for Go runtimes.
        :param pulumi.Input[bool] noop_on_destroy: If set to `true`, the application version will not be deleted.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] readiness_check: Configures readiness health checking for instances. Unhealthy instances are not put into the backend traffic rotation.  Structure is documented below.
        :param pulumi.Input[dict] resources: Machine resources for a version.  Structure is documented below.
        :param pulumi.Input[str] runtime: Desired runtime. Example python27.
        :param pulumi.Input[str] runtime_api_version: The version of the API in the given runtime environment.
               Please see the app.yaml reference for valid values at https://cloud.google.com/appengine/docs/standard//config/appref
        :param pulumi.Input[str] runtime_channel: The channel of the runtime to use. Only available for some runtimes.
        :param pulumi.Input[str] runtime_main_executable_path: The path or name of the app's main executable.
        :param pulumi.Input[str] service: AppEngine service resource
        :param pulumi.Input[str] serving_status: Current serving status of this version. Only the versions with a SERVING status create instances and can be billed.
        :param pulumi.Input[str] version_id: Relative name of the version within the service. For example, `v1`. Version names can contain only lowercase letters, numbers, or hyphens.
               Reserved names,"default", "latest", and any name with the prefix "ah-".
        :param pulumi.Input[dict] vpc_access_connector: Enables VPC connectivity for standard apps.  Structure is documented below.

        The **api_config** object supports the following:

          * `authFailAction` (`pulumi.Input[str]`) - Action to take when users access resources that require authentication.
          * `login` (`pulumi.Input[str]`) - Level of login required to access this resource.
          * `script` (`pulumi.Input[str]`) - Path to the script from the application root directory.
          * `securityLevel` (`pulumi.Input[str]`) - Security (HTTPS) enforcement for this URL.
          * `url` (`pulumi.Input[str]`) - URL to serve the endpoint at.

        The **automatic_scaling** object supports the following:

          * `coolDownPeriod` (`pulumi.Input[str]`) - The time period that the Autoscaler should wait before it starts collecting information from a new instance.
            This prevents the autoscaler from collecting information when the instance is initializing,
            during which the collected usage would not be reliable. Default: 120s
          * `cpuUtilization` (`pulumi.Input[dict]`) - Target scaling by CPU usage.  Structure is documented below.
            * `aggregationWindowLength` (`pulumi.Input[str]`) - Period of time over which CPU utilization is calculated.
            * `targetUtilization` (`pulumi.Input[float]`) - Target CPU utilization ratio to maintain when scaling. Must be between 0 and 1.

          * `diskUtilization` (`pulumi.Input[dict]`) - Target scaling by disk usage.  Structure is documented below.
            * `targetReadBytesPerSecond` (`pulumi.Input[float]`) - Target bytes read per second.
            * `targetReadOpsPerSecond` (`pulumi.Input[float]`) - Target ops read per seconds.
            * `targetWriteBytesPerSecond` (`pulumi.Input[float]`) - Target bytes written per second.
            * `targetWriteOpsPerSecond` (`pulumi.Input[float]`) - Target ops written per second.

          * `maxConcurrentRequests` (`pulumi.Input[float]`) - Number of concurrent requests an automatic scaling instance can accept before the scheduler spawns a new instance.
            Defaults to a runtime-specific value.
          * `maxIdleInstances` (`pulumi.Input[float]`) - Maximum number of idle instances that should be maintained for this version.
          * `maxPendingLatency` (`pulumi.Input[str]`) - Maximum amount of time that a request should wait in the pending queue before starting a new instance to handle it.
          * `maxTotalInstances` (`pulumi.Input[float]`) - Maximum number of instances that should be started to handle requests for this version. Default: 20
          * `minIdleInstances` (`pulumi.Input[float]`) - Minimum number of idle instances that should be maintained for this version. Only applicable for the default version of a service.
          * `minPendingLatency` (`pulumi.Input[str]`) - Minimum amount of time a request should wait in the pending queue before starting a new instance to handle it.
          * `minTotalInstances` (`pulumi.Input[float]`) - Minimum number of running instances that should be maintained for this version. Default: 2
          * `networkUtilization` (`pulumi.Input[dict]`) - Target scaling by network usage.  Structure is documented below.
            * `targetReceivedBytesPerSecond` (`pulumi.Input[float]`) - Target bytes received per second.
            * `targetReceivedPacketsPerSecond` (`pulumi.Input[float]`) - Target packets received per second.
            * `targetSentBytesPerSecond` (`pulumi.Input[float]`) - Target bytes sent per second.
            * `targetSentPacketsPerSecond` (`pulumi.Input[float]`) - Target packets sent per second.

          * `requestUtilization` (`pulumi.Input[dict]`) - Target scaling by request utilization.  Structure is documented below.
            * `targetConcurrentRequests` (`pulumi.Input[float]`) - Target number of concurrent requests.
            * `targetRequestCountPerSecond` (`pulumi.Input[str]`) - Target requests per second.

        The **deployment** object supports the following:

          * `cloudBuildOptions` (`pulumi.Input[dict]`) - Options for the build operations performed as a part of the version deployment. Only applicable when creating a version using source code directly.  Structure is documented below.
            * `appYamlPath` (`pulumi.Input[str]`) - Path to the yaml file used in deployment, used to determine runtime configuration details.
            * `cloudBuildTimeout` (`pulumi.Input[str]`) - The Cloud Build timeout used as part of any dependent builds performed by version creation. Defaults to 10 minutes.
              A duration in seconds with up to nine fractional digits, terminated by 's'. Example: "3.5s".

          * `container` (`pulumi.Input[dict]`) - The Docker image for the container that runs the version.  Structure is documented below.
            * `image` (`pulumi.Input[str]`) - URI to the hosted container image in Google Container Registry. The URI must be fully qualified and include a tag or digest.
              Examples: "gcr.io/my-project/image:tag" or "gcr.io/my-project/image@digest"

          * `files` (`pulumi.Input[list]`) - Manifest of the files stored in Google Cloud Storage that are included as part of this version.
            All files must be readable using the credentials supplied with this call.  Structure is documented below.
            * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
            * `sha1Sum` (`pulumi.Input[str]`) - SHA1 checksum of the file
            * `sourceUrl` (`pulumi.Input[str]`) - Source URL

          * `zip` (`pulumi.Input[dict]`) - Zip File  Structure is documented below.
            * `filesCount` (`pulumi.Input[float]`) - files count
            * `sourceUrl` (`pulumi.Input[str]`) - Source URL

        The **endpoints_api_service** object supports the following:

          * `config_id` (`pulumi.Input[str]`) - Endpoints service configuration ID as specified by the Service Management API. For example "2016-09-19r1".
            By default, the rollout strategy for Endpoints is "FIXED". This means that Endpoints starts up with a particular configuration ID.
            When a new configuration is rolled out, Endpoints must be given the new configuration ID. The configId field is used to give the configuration ID
            and is required in this case.
            Endpoints also has a rollout strategy called "MANAGED". When using this, Endpoints fetches the latest configuration and does not need
            the configuration ID. In this case, configId must be omitted.
          * `disableTraceSampling` (`pulumi.Input[bool]`) - Enable or disable trace sampling. By default, this is set to false for enabled.
          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
          * `rolloutStrategy` (`pulumi.Input[str]`) - Endpoints rollout strategy. If FIXED, configId must be specified. If MANAGED, configId must be omitted.

        The **entrypoint** object supports the following:

          * `shell` (`pulumi.Input[str]`) - The format should be a shell command that can be fed to bash -c.

        The **handlers** object supports the following:

          * `authFailAction` (`pulumi.Input[str]`) - Action to take when users access resources that require authentication.
          * `login` (`pulumi.Input[str]`) - Level of login required to access this resource.
          * `redirectHttpResponseCode` (`pulumi.Input[str]`) - 30x code to use when performing redirects for the secure field.
          * `script` (`pulumi.Input[dict]`) - Path to the script from the application root directory.
            * `scriptPath` (`pulumi.Input[str]`) - Path to the script from the application root directory.

          * `securityLevel` (`pulumi.Input[str]`) - Security (HTTPS) enforcement for this URL.
          * `staticFiles` (`pulumi.Input[dict]`) - Files served directly to the user for a given URL, such as images, CSS stylesheets, or JavaScript source files.
            Static file handlers describe which files in the application directory are static files, and which URLs serve them.  Structure is documented below.
            * `applicationReadable` (`pulumi.Input[bool]`) - Whether files should also be uploaded as code data. By default, files declared in static file handlers are
              uploaded as static data and are only served to end users; they cannot be read by the application. If enabled,
              uploads are charged against both your code and static data storage resource quotas.
            * `expiration` (`pulumi.Input[str]`) - Time a static file served by this handler should be cached by web proxies and browsers.
              A duration in seconds with up to nine fractional digits, terminated by 's'. Example "3.5s".
              Default is '0s'
            * `httpHeaders` (`pulumi.Input[dict]`) - HTTP headers to use for all responses from these URLs.
              An object containing a list of "key:value" value pairs.".
            * `mimeType` (`pulumi.Input[str]`) - MIME type used to serve all files served by this handler.
              Defaults to file-specific MIME types, which are derived from each file's filename extension.
            * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
              The path can refer to text matched in groupings in the URL pattern.
            * `requireMatchingFile` (`pulumi.Input[bool]`) - Whether this handler should match the request if the file referenced by the handler does not exist.
            * `uploadPathRegex` (`pulumi.Input[str]`) - Regular expression that matches the file paths for all files that should be referenced by this handler.

          * `urlRegex` (`pulumi.Input[str]`) - URL prefix. Uses regular expression syntax, which means regexp special characters must be escaped, but should not contain groupings.
            All URLs that begin with this prefix are handled by this handler, using the portion of the URL after the prefix as part of the file path.

        The **liveness_check** object supports the following:

          * `checkInterval` (`pulumi.Input[str]`) - Interval between health checks.
          * `failureThreshold` (`pulumi.Input[float]`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
          * `host` (`pulumi.Input[str]`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
          * `initialDelay` (`pulumi.Input[str]`) - The initial delay before starting to execute the checks. Default: "300s"
          * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
            The path can refer to text matched in groupings in the URL pattern.
          * `successThreshold` (`pulumi.Input[float]`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
          * `timeout` (`pulumi.Input[str]`) - Time before the check is considered failed. Default: "4s"

        The **manual_scaling** object supports the following:

          * `instances` (`pulumi.Input[float]`) - Number of instances to assign to the service at the start.
            **Note:** When managing the number of instances at runtime through the App Engine Admin API or the (now deprecated) Python 2
            Modules API set_num_instances() you must use `lifecycle.ignore_changes = ["manual_scaling"[0].instances]` to prevent drift detection.

        The **network** object supports the following:

          * `forwardedPorts` (`pulumi.Input[list]`) - List of ports, or port pairs, to forward from the virtual machine to the application container.
          * `instanceTag` (`pulumi.Input[str]`) - Tag to apply to the instance during creation.
          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
          * `session_affinity` (`pulumi.Input[bool]`) - Enable session affinity.
          * `subnetwork` (`pulumi.Input[str]`) - Google Cloud Platform sub-network where the virtual machines are created. Specify the short name, not the resource path.
            If the network that the instance is being created in is a Legacy network, then the IP address is allocated from the IPv4Range.
            If the network that the instance is being created in is an auto Subnet Mode Network, then only network name should be specified (not the subnetworkName) and the IP address is created from the IPCidrRange of the subnetwork that exists in that zone for that network.
            If the network that the instance is being created in is a custom Subnet Mode Network, then the subnetworkName must be specified and the IP address is created from the IPCidrRange of the subnetwork.
            If specified, the subnetwork must exist in the same region as the App Engine flexible environment application.

        The **readiness_check** object supports the following:

          * `appStartTimeout` (`pulumi.Input[str]`) - A maximum time limit on application initialization, measured from moment the application successfully
            replies to a healthcheck until it is ready to serve traffic. Default: "300s"
          * `checkInterval` (`pulumi.Input[str]`) - Interval between health checks.
          * `failureThreshold` (`pulumi.Input[float]`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
          * `host` (`pulumi.Input[str]`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
          * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
            The path can refer to text matched in groupings in the URL pattern.
          * `successThreshold` (`pulumi.Input[float]`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
          * `timeout` (`pulumi.Input[str]`) - Time before the check is considered failed. Default: "4s"

        The **resources** object supports the following:

          * `cpu` (`pulumi.Input[float]`) - Number of CPU cores needed.
          * `diskGb` (`pulumi.Input[float]`) - Disk size (GB) needed.
          * `memoryGb` (`pulumi.Input[float]`) - Memory (GB) needed.
          * `volumes` (`pulumi.Input[list]`) - List of ports, or port pairs, to forward from the virtual machine to the application container.  Structure is documented below.
            * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
            * `sizeGb` (`pulumi.Input[float]`) - Volume size in gigabytes.
            * `volumeType` (`pulumi.Input[str]`) - Underlying volume type, e.g. 'tmpfs'.

        The **vpc_access_connector** object supports the following:

          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['api_config'] = api_config
            __props__['automatic_scaling'] = automatic_scaling
            __props__['beta_settings'] = beta_settings
            __props__['default_expiration'] = default_expiration
            __props__['delete_service_on_destroy'] = delete_service_on_destroy
            __props__['deployment'] = deployment
            __props__['endpoints_api_service'] = endpoints_api_service
            __props__['entrypoint'] = entrypoint
            __props__['env_variables'] = env_variables
            __props__['handlers'] = handlers
            __props__['inbound_services'] = inbound_services
            __props__['instance_class'] = instance_class
            if liveness_check is None:
                raise TypeError("Missing required property 'liveness_check'")
            __props__['liveness_check'] = liveness_check
            __props__['manual_scaling'] = manual_scaling
            __props__['network'] = network
            __props__['nobuild_files_regex'] = nobuild_files_regex
            __props__['noop_on_destroy'] = noop_on_destroy
            __props__['project'] = project
            if readiness_check is None:
                raise TypeError("Missing required property 'readiness_check'")
            __props__['readiness_check'] = readiness_check
            __props__['resources'] = resources
            if runtime is None:
                raise TypeError("Missing required property 'runtime'")
            __props__['runtime'] = runtime
            __props__['runtime_api_version'] = runtime_api_version
            __props__['runtime_channel'] = runtime_channel
            __props__['runtime_main_executable_path'] = runtime_main_executable_path
            __props__['service'] = service
            __props__['serving_status'] = serving_status
            __props__['version_id'] = version_id
            __props__['vpc_access_connector'] = vpc_access_connector
            __props__['name'] = None
        super(FlexibleAppVersion, __self__).__init__(
            'gcp:appengine/flexibleAppVersion:FlexibleAppVersion',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, api_config=None, automatic_scaling=None, beta_settings=None, default_expiration=None, delete_service_on_destroy=None, deployment=None, endpoints_api_service=None, entrypoint=None, env_variables=None, handlers=None, inbound_services=None, instance_class=None, liveness_check=None, manual_scaling=None, name=None, network=None, nobuild_files_regex=None, noop_on_destroy=None, project=None, readiness_check=None, resources=None, runtime=None, runtime_api_version=None, runtime_channel=None, runtime_main_executable_path=None, service=None, serving_status=None, version_id=None, vpc_access_connector=None):
        """
        Get an existing FlexibleAppVersion resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] api_config: Serving configuration for Google Cloud Endpoints.  Structure is documented below.
        :param pulumi.Input[dict] automatic_scaling: Automatic scaling is based on request rate, response latencies, and other application metrics.  Structure is documented below.
        :param pulumi.Input[dict] beta_settings: Metadata settings that are supplied to this version to enable beta runtime features.
        :param pulumi.Input[str] default_expiration: Duration that static files should be cached by web proxies and browsers.
               Only applicable if the corresponding StaticFilesHandler does not specify its own expiration time.
        :param pulumi.Input[bool] delete_service_on_destroy: If set to `true`, the service will be deleted if it is the last version.
        :param pulumi.Input[dict] deployment: Code and application artifacts that make up this version.  Structure is documented below.
        :param pulumi.Input[dict] endpoints_api_service: Code and application artifacts that make up this version.  Structure is documented below.
        :param pulumi.Input[dict] entrypoint: The entrypoint for the application.  Structure is documented below.
        :param pulumi.Input[dict] env_variables: Environment variables available to the application.  As these are not returned in the API request, the provider will not detect any changes made outside of the config.
        :param pulumi.Input[list] handlers: An ordered list of URL-matching patterns that should be applied to incoming requests.
               The first matching URL handles the request and other request handlers are not attempted.  Structure is documented below.
        :param pulumi.Input[list] inbound_services: Before an application can receive email or XMPP messages, the application must be configured to enable the service.
        :param pulumi.Input[str] instance_class: Instance class that is used to run this version. Valid values are
               AutomaticScaling: F1, F2, F4, F4_1G
               ManualScaling: B1, B2, B4, B8, B4_1G
               Defaults to F1 for AutomaticScaling and B1 for ManualScaling.
        :param pulumi.Input[dict] liveness_check: Health checking configuration for VM instances. Unhealthy instances are killed and replaced with new instances.  Structure is documented below.
        :param pulumi.Input[dict] manual_scaling: A service with manual scaling runs continuously, allowing you to perform complex initialization and rely on the state of its memory over time.  Structure is documented below.
        :param pulumi.Input[str] name: Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
        :param pulumi.Input[dict] network: Extra network settings  Structure is documented below.
        :param pulumi.Input[str] nobuild_files_regex: Files that match this pattern will not be built into this version. Only applicable for Go runtimes.
        :param pulumi.Input[bool] noop_on_destroy: If set to `true`, the application version will not be deleted.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] readiness_check: Configures readiness health checking for instances. Unhealthy instances are not put into the backend traffic rotation.  Structure is documented below.
        :param pulumi.Input[dict] resources: Machine resources for a version.  Structure is documented below.
        :param pulumi.Input[str] runtime: Desired runtime. Example python27.
        :param pulumi.Input[str] runtime_api_version: The version of the API in the given runtime environment.
               Please see the app.yaml reference for valid values at https://cloud.google.com/appengine/docs/standard//config/appref
        :param pulumi.Input[str] runtime_channel: The channel of the runtime to use. Only available for some runtimes.
        :param pulumi.Input[str] runtime_main_executable_path: The path or name of the app's main executable.
        :param pulumi.Input[str] service: AppEngine service resource
        :param pulumi.Input[str] serving_status: Current serving status of this version. Only the versions with a SERVING status create instances and can be billed.
        :param pulumi.Input[str] version_id: Relative name of the version within the service. For example, `v1`. Version names can contain only lowercase letters, numbers, or hyphens.
               Reserved names,"default", "latest", and any name with the prefix "ah-".
        :param pulumi.Input[dict] vpc_access_connector: Enables VPC connectivity for standard apps.  Structure is documented below.

        The **api_config** object supports the following:

          * `authFailAction` (`pulumi.Input[str]`) - Action to take when users access resources that require authentication.
          * `login` (`pulumi.Input[str]`) - Level of login required to access this resource.
          * `script` (`pulumi.Input[str]`) - Path to the script from the application root directory.
          * `securityLevel` (`pulumi.Input[str]`) - Security (HTTPS) enforcement for this URL.
          * `url` (`pulumi.Input[str]`) - URL to serve the endpoint at.

        The **automatic_scaling** object supports the following:

          * `coolDownPeriod` (`pulumi.Input[str]`) - The time period that the Autoscaler should wait before it starts collecting information from a new instance.
            This prevents the autoscaler from collecting information when the instance is initializing,
            during which the collected usage would not be reliable. Default: 120s
          * `cpuUtilization` (`pulumi.Input[dict]`) - Target scaling by CPU usage.  Structure is documented below.
            * `aggregationWindowLength` (`pulumi.Input[str]`) - Period of time over which CPU utilization is calculated.
            * `targetUtilization` (`pulumi.Input[float]`) - Target CPU utilization ratio to maintain when scaling. Must be between 0 and 1.

          * `diskUtilization` (`pulumi.Input[dict]`) - Target scaling by disk usage.  Structure is documented below.
            * `targetReadBytesPerSecond` (`pulumi.Input[float]`) - Target bytes read per second.
            * `targetReadOpsPerSecond` (`pulumi.Input[float]`) - Target ops read per seconds.
            * `targetWriteBytesPerSecond` (`pulumi.Input[float]`) - Target bytes written per second.
            * `targetWriteOpsPerSecond` (`pulumi.Input[float]`) - Target ops written per second.

          * `maxConcurrentRequests` (`pulumi.Input[float]`) - Number of concurrent requests an automatic scaling instance can accept before the scheduler spawns a new instance.
            Defaults to a runtime-specific value.
          * `maxIdleInstances` (`pulumi.Input[float]`) - Maximum number of idle instances that should be maintained for this version.
          * `maxPendingLatency` (`pulumi.Input[str]`) - Maximum amount of time that a request should wait in the pending queue before starting a new instance to handle it.
          * `maxTotalInstances` (`pulumi.Input[float]`) - Maximum number of instances that should be started to handle requests for this version. Default: 20
          * `minIdleInstances` (`pulumi.Input[float]`) - Minimum number of idle instances that should be maintained for this version. Only applicable for the default version of a service.
          * `minPendingLatency` (`pulumi.Input[str]`) - Minimum amount of time a request should wait in the pending queue before starting a new instance to handle it.
          * `minTotalInstances` (`pulumi.Input[float]`) - Minimum number of running instances that should be maintained for this version. Default: 2
          * `networkUtilization` (`pulumi.Input[dict]`) - Target scaling by network usage.  Structure is documented below.
            * `targetReceivedBytesPerSecond` (`pulumi.Input[float]`) - Target bytes received per second.
            * `targetReceivedPacketsPerSecond` (`pulumi.Input[float]`) - Target packets received per second.
            * `targetSentBytesPerSecond` (`pulumi.Input[float]`) - Target bytes sent per second.
            * `targetSentPacketsPerSecond` (`pulumi.Input[float]`) - Target packets sent per second.

          * `requestUtilization` (`pulumi.Input[dict]`) - Target scaling by request utilization.  Structure is documented below.
            * `targetConcurrentRequests` (`pulumi.Input[float]`) - Target number of concurrent requests.
            * `targetRequestCountPerSecond` (`pulumi.Input[str]`) - Target requests per second.

        The **deployment** object supports the following:

          * `cloudBuildOptions` (`pulumi.Input[dict]`) - Options for the build operations performed as a part of the version deployment. Only applicable when creating a version using source code directly.  Structure is documented below.
            * `appYamlPath` (`pulumi.Input[str]`) - Path to the yaml file used in deployment, used to determine runtime configuration details.
            * `cloudBuildTimeout` (`pulumi.Input[str]`) - The Cloud Build timeout used as part of any dependent builds performed by version creation. Defaults to 10 minutes.
              A duration in seconds with up to nine fractional digits, terminated by 's'. Example: "3.5s".

          * `container` (`pulumi.Input[dict]`) - The Docker image for the container that runs the version.  Structure is documented below.
            * `image` (`pulumi.Input[str]`) - URI to the hosted container image in Google Container Registry. The URI must be fully qualified and include a tag or digest.
              Examples: "gcr.io/my-project/image:tag" or "gcr.io/my-project/image@digest"

          * `files` (`pulumi.Input[list]`) - Manifest of the files stored in Google Cloud Storage that are included as part of this version.
            All files must be readable using the credentials supplied with this call.  Structure is documented below.
            * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
            * `sha1Sum` (`pulumi.Input[str]`) - SHA1 checksum of the file
            * `sourceUrl` (`pulumi.Input[str]`) - Source URL

          * `zip` (`pulumi.Input[dict]`) - Zip File  Structure is documented below.
            * `filesCount` (`pulumi.Input[float]`) - files count
            * `sourceUrl` (`pulumi.Input[str]`) - Source URL

        The **endpoints_api_service** object supports the following:

          * `config_id` (`pulumi.Input[str]`) - Endpoints service configuration ID as specified by the Service Management API. For example "2016-09-19r1".
            By default, the rollout strategy for Endpoints is "FIXED". This means that Endpoints starts up with a particular configuration ID.
            When a new configuration is rolled out, Endpoints must be given the new configuration ID. The configId field is used to give the configuration ID
            and is required in this case.
            Endpoints also has a rollout strategy called "MANAGED". When using this, Endpoints fetches the latest configuration and does not need
            the configuration ID. In this case, configId must be omitted.
          * `disableTraceSampling` (`pulumi.Input[bool]`) - Enable or disable trace sampling. By default, this is set to false for enabled.
          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
          * `rolloutStrategy` (`pulumi.Input[str]`) - Endpoints rollout strategy. If FIXED, configId must be specified. If MANAGED, configId must be omitted.

        The **entrypoint** object supports the following:

          * `shell` (`pulumi.Input[str]`) - The format should be a shell command that can be fed to bash -c.

        The **handlers** object supports the following:

          * `authFailAction` (`pulumi.Input[str]`) - Action to take when users access resources that require authentication.
          * `login` (`pulumi.Input[str]`) - Level of login required to access this resource.
          * `redirectHttpResponseCode` (`pulumi.Input[str]`) - 30x code to use when performing redirects for the secure field.
          * `script` (`pulumi.Input[dict]`) - Path to the script from the application root directory.
            * `scriptPath` (`pulumi.Input[str]`) - Path to the script from the application root directory.

          * `securityLevel` (`pulumi.Input[str]`) - Security (HTTPS) enforcement for this URL.
          * `staticFiles` (`pulumi.Input[dict]`) - Files served directly to the user for a given URL, such as images, CSS stylesheets, or JavaScript source files.
            Static file handlers describe which files in the application directory are static files, and which URLs serve them.  Structure is documented below.
            * `applicationReadable` (`pulumi.Input[bool]`) - Whether files should also be uploaded as code data. By default, files declared in static file handlers are
              uploaded as static data and are only served to end users; they cannot be read by the application. If enabled,
              uploads are charged against both your code and static data storage resource quotas.
            * `expiration` (`pulumi.Input[str]`) - Time a static file served by this handler should be cached by web proxies and browsers.
              A duration in seconds with up to nine fractional digits, terminated by 's'. Example "3.5s".
              Default is '0s'
            * `httpHeaders` (`pulumi.Input[dict]`) - HTTP headers to use for all responses from these URLs.
              An object containing a list of "key:value" value pairs.".
            * `mimeType` (`pulumi.Input[str]`) - MIME type used to serve all files served by this handler.
              Defaults to file-specific MIME types, which are derived from each file's filename extension.
            * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
              The path can refer to text matched in groupings in the URL pattern.
            * `requireMatchingFile` (`pulumi.Input[bool]`) - Whether this handler should match the request if the file referenced by the handler does not exist.
            * `uploadPathRegex` (`pulumi.Input[str]`) - Regular expression that matches the file paths for all files that should be referenced by this handler.

          * `urlRegex` (`pulumi.Input[str]`) - URL prefix. Uses regular expression syntax, which means regexp special characters must be escaped, but should not contain groupings.
            All URLs that begin with this prefix are handled by this handler, using the portion of the URL after the prefix as part of the file path.

        The **liveness_check** object supports the following:

          * `checkInterval` (`pulumi.Input[str]`) - Interval between health checks.
          * `failureThreshold` (`pulumi.Input[float]`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
          * `host` (`pulumi.Input[str]`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
          * `initialDelay` (`pulumi.Input[str]`) - The initial delay before starting to execute the checks. Default: "300s"
          * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
            The path can refer to text matched in groupings in the URL pattern.
          * `successThreshold` (`pulumi.Input[float]`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
          * `timeout` (`pulumi.Input[str]`) - Time before the check is considered failed. Default: "4s"

        The **manual_scaling** object supports the following:

          * `instances` (`pulumi.Input[float]`) - Number of instances to assign to the service at the start.
            **Note:** When managing the number of instances at runtime through the App Engine Admin API or the (now deprecated) Python 2
            Modules API set_num_instances() you must use `lifecycle.ignore_changes = ["manual_scaling"[0].instances]` to prevent drift detection.

        The **network** object supports the following:

          * `forwardedPorts` (`pulumi.Input[list]`) - List of ports, or port pairs, to forward from the virtual machine to the application container.
          * `instanceTag` (`pulumi.Input[str]`) - Tag to apply to the instance during creation.
          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
          * `session_affinity` (`pulumi.Input[bool]`) - Enable session affinity.
          * `subnetwork` (`pulumi.Input[str]`) - Google Cloud Platform sub-network where the virtual machines are created. Specify the short name, not the resource path.
            If the network that the instance is being created in is a Legacy network, then the IP address is allocated from the IPv4Range.
            If the network that the instance is being created in is an auto Subnet Mode Network, then only network name should be specified (not the subnetworkName) and the IP address is created from the IPCidrRange of the subnetwork that exists in that zone for that network.
            If the network that the instance is being created in is a custom Subnet Mode Network, then the subnetworkName must be specified and the IP address is created from the IPCidrRange of the subnetwork.
            If specified, the subnetwork must exist in the same region as the App Engine flexible environment application.

        The **readiness_check** object supports the following:

          * `appStartTimeout` (`pulumi.Input[str]`) - A maximum time limit on application initialization, measured from moment the application successfully
            replies to a healthcheck until it is ready to serve traffic. Default: "300s"
          * `checkInterval` (`pulumi.Input[str]`) - Interval between health checks.
          * `failureThreshold` (`pulumi.Input[float]`) - Number of consecutive failed checks required before considering the VM unhealthy. Default: 4.
          * `host` (`pulumi.Input[str]`) - Host header to send when performing a HTTP Readiness check. Example: "myapp.appspot.com"
          * `path` (`pulumi.Input[str]`) - Path to the static files matched by the URL pattern, from the application root directory.
            The path can refer to text matched in groupings in the URL pattern.
          * `successThreshold` (`pulumi.Input[float]`) - Number of consecutive successful checks required before considering the VM healthy. Default: 2.
          * `timeout` (`pulumi.Input[str]`) - Time before the check is considered failed. Default: "4s"

        The **resources** object supports the following:

          * `cpu` (`pulumi.Input[float]`) - Number of CPU cores needed.
          * `diskGb` (`pulumi.Input[float]`) - Disk size (GB) needed.
          * `memoryGb` (`pulumi.Input[float]`) - Memory (GB) needed.
          * `volumes` (`pulumi.Input[list]`) - List of ports, or port pairs, to forward from the virtual machine to the application container.  Structure is documented below.
            * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
            * `sizeGb` (`pulumi.Input[float]`) - Volume size in gigabytes.
            * `volumeType` (`pulumi.Input[str]`) - Underlying volume type, e.g. 'tmpfs'.

        The **vpc_access_connector** object supports the following:

          * `name` (`pulumi.Input[str]`) - Full Serverless VPC Access Connector name e.g. /projects/my-project/locations/us-central1/connectors/c1.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["api_config"] = api_config
        __props__["automatic_scaling"] = automatic_scaling
        __props__["beta_settings"] = beta_settings
        __props__["default_expiration"] = default_expiration
        __props__["delete_service_on_destroy"] = delete_service_on_destroy
        __props__["deployment"] = deployment
        __props__["endpoints_api_service"] = endpoints_api_service
        __props__["entrypoint"] = entrypoint
        __props__["env_variables"] = env_variables
        __props__["handlers"] = handlers
        __props__["inbound_services"] = inbound_services
        __props__["instance_class"] = instance_class
        __props__["liveness_check"] = liveness_check
        __props__["manual_scaling"] = manual_scaling
        __props__["name"] = name
        __props__["network"] = network
        __props__["nobuild_files_regex"] = nobuild_files_regex
        __props__["noop_on_destroy"] = noop_on_destroy
        __props__["project"] = project
        __props__["readiness_check"] = readiness_check
        __props__["resources"] = resources
        __props__["runtime"] = runtime
        __props__["runtime_api_version"] = runtime_api_version
        __props__["runtime_channel"] = runtime_channel
        __props__["runtime_main_executable_path"] = runtime_main_executable_path
        __props__["service"] = service
        __props__["serving_status"] = serving_status
        __props__["version_id"] = version_id
        __props__["vpc_access_connector"] = vpc_access_connector
        return FlexibleAppVersion(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Bucket(pulumi.CustomResource):
    bucket_policy_only: pulumi.Output[bool]
    """
    Enables [Bucket Policy Only](https://cloud.google.com/storage/docs/bucket-policy-only) access to a bucket.
    """
    cors: pulumi.Output[list]
    """
    The bucket's [Cross-Origin Resource Sharing (CORS)](https://www.w3.org/TR/cors/) configuration. Multiple blocks of this type are permitted. Structure is documented below.

      * `maxAgeSeconds` (`float`) - The value, in seconds, to return in the [Access-Control-Max-Age header](https://www.w3.org/TR/cors/#access-control-max-age-response-header) used in preflight responses.
      * `methods` (`list`) - The list of HTTP methods on which to include CORS response headers, (GET, OPTIONS, POST, etc) Note: "*" is permitted in the list of methods, and means "any method".
      * `origins` (`list`) - The list of [Origins](https://tools.ietf.org/html/rfc6454) eligible to receive CORS response headers. Note: "*" is permitted in the list of origins, and means "any Origin".
      * `responseHeaders` (`list`) - The list of HTTP headers other than the [simple response headers](https://www.w3.org/TR/cors/#simple-response-header) to give permission for the user-agent to share across domains.
    """
    default_event_based_hold: pulumi.Output[bool]
    encryption: pulumi.Output[dict]
    """
    The bucket's encryption configuration.

      * `defaultKmsKeyName` (`str`)
    """
    force_destroy: pulumi.Output[bool]
    """
    When deleting a bucket, this
    boolean option will delete all contained objects. If you try to delete a
    bucket that contains objects, the provider will fail that run.
    """
    labels: pulumi.Output[dict]
    """
    A set of key/value label pairs to assign to the bucket.
    """
    lifecycle_rules: pulumi.Output[list]
    """
    The bucket's [Lifecycle Rules](https://cloud.google.com/storage/docs/lifecycle#configuration) configuration. Multiple blocks of this type are permitted. Structure is documented below.

      * `action` (`dict`) - The Lifecycle Rule's action configuration. A single block of this type is supported. Structure is documented below.
        * `storage_class` (`str`) - The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
        * `type` (`str`) - The type of the action of this Lifecycle Rule. Supported values include: `Delete` and `SetStorageClass`.

      * `condition` (`dict`) - The Lifecycle Rule's condition configuration. A single block of this type is supported. Structure is documented below.
        * `age` (`float`) - Minimum age of an object in days to satisfy this condition.
        * `createdBefore` (`str`) - Creation date of an object in RFC 3339 (e.g. `2017-06-13`) to satisfy this condition.
        * `matchesStorageClasses` (`list`) - [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects to satisfy this condition. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`, `STANDARD`, `DURABLE_REDUCED_AVAILABILITY`.
        * `numNewerVersions` (`float`) - Relevant only for versioned objects. The number of newer versions of an object to satisfy this condition.
        * `withState` (`str`) - Match to live and/or archived objects. Unversioned buckets have only live objects. Supported values include: `"LIVE"`, `"ARCHIVED"`, `"ANY"`.
    """
    location: pulumi.Output[str]
    """
    The [GCS location](https://cloud.google.com/storage/docs/bucket-locations)
    """
    logging: pulumi.Output[dict]
    """
    The bucket's [Access & Storage Logs](https://cloud.google.com/storage/docs/access-logs) configuration.

      * `logBucket` (`str`) - The bucket that will receive log objects.
      * `logObjectPrefix` (`str`) - The object prefix for log objects. If it's not provided,
        by default GCS sets this to this bucket's name.
    """
    name: pulumi.Output[str]
    """
    The name of the bucket.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    requester_pays: pulumi.Output[bool]
    """
    Enables [Requester Pays](https://cloud.google.com/storage/docs/requester-pays) on a storage bucket.
    """
    retention_policy: pulumi.Output[dict]
    """
    Configuration of the bucket's data retention policy for how long objects in the bucket should be retained. Structure is documented below.

      * `isLocked` (`bool`) - If set to `true`, the bucket will be [locked](https://cloud.google.com/storage/docs/using-bucket-lock#lock-bucket) and permanently restrict edits to the bucket's retention policy.  Caution: Locking a bucket is an irreversible action.
      * `retentionPeriod` (`float`) - The period of time, in seconds, that objects in the bucket must be retained and cannot be deleted, overwritten, or archived. The value must be less than 3,155,760,000 seconds.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    storage_class: pulumi.Output[str]
    """
    The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
    """
    url: pulumi.Output[str]
    """
    The base URL of the bucket, in the format `gs://<bucket-name>`.
    """
    versioning: pulumi.Output[dict]
    """
    The bucket's [Versioning](https://cloud.google.com/storage/docs/object-versioning) configuration.

      * `enabled` (`bool`) - While set to `true`, versioning is fully enabled for this bucket.
    """
    website: pulumi.Output[dict]
    """
    Configuration if the bucket acts as a website. Structure is documented below.

      * `mainPageSuffix` (`str`) - Behaves as the bucket's directory index where
        missing objects are treated as potential directories.
      * `notFoundPage` (`str`) - The custom object to return when a requested
        resource is not found.
    """
    def __init__(__self__, resource_name, opts=None, bucket_policy_only=None, cors=None, default_event_based_hold=None, encryption=None, force_destroy=None, labels=None, lifecycle_rules=None, location=None, logging=None, name=None, project=None, requester_pays=None, retention_policy=None, storage_class=None, versioning=None, website=None, __props__=None, __name__=None, __opts__=None):
        """
        Creates a new bucket in Google cloud storage service (GCS).
        Once a bucket has been created, its location can't be changed.
        [ACLs](https://cloud.google.com/storage/docs/access-control/lists) can be applied
        using the [`storage.BucketACL`](https://www.terraform.io/docs/providers/google/r/storage_bucket_acl.html) resource.

        For more information see
        [the official documentation](https://cloud.google.com/storage/docs/overview)
        and
        [API](https://cloud.google.com/storage/docs/json_api/v1/buckets).

        **Note**: If the project id is not set on the resource or in the provider block it will be dynamically
        determined which will require enabling the compute api.


        ## Example Usage

        ### Creating A Private Bucket In Standard Storage, In The EU Region. Bucket Configured As Static Website And CORS Configurations

        ```python
        import pulumi
        import pulumi_gcp as gcp

        static_site = gcp.storage.Bucket("static-site",
            bucket_policy_only=True,
            cors=[{
                "maxAgeSeconds": 3600,
                "method": [
                    "GET",
                    "HEAD",
                    "PUT",
                    "POST",
                    "DELETE",
                ],
                "origin": ["http://image-store.com"],
                "responseHeader": ["*"],
            }],
            force_destroy=True,
            location="EU",
            website={
                "mainPageSuffix": "index.html",
                "notFoundPage": "404.html",
            })
        ```

        ### Life Cycle Settings For Storage Bucket Objects

        ```python
        import pulumi
        import pulumi_gcp as gcp

        auto_expire = gcp.storage.Bucket("auto-expire",
            force_destroy=True,
            lifecycle_rules=[{
                "action": {
                    "type": "Delete",
                },
                "condition": {
                    "age": "3",
                },
            }],
            location="US")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] bucket_policy_only: Enables [Bucket Policy Only](https://cloud.google.com/storage/docs/bucket-policy-only) access to a bucket.
        :param pulumi.Input[list] cors: The bucket's [Cross-Origin Resource Sharing (CORS)](https://www.w3.org/TR/cors/) configuration. Multiple blocks of this type are permitted. Structure is documented below.
        :param pulumi.Input[dict] encryption: The bucket's encryption configuration.
        :param pulumi.Input[bool] force_destroy: When deleting a bucket, this
               boolean option will delete all contained objects. If you try to delete a
               bucket that contains objects, the provider will fail that run.
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to the bucket.
        :param pulumi.Input[list] lifecycle_rules: The bucket's [Lifecycle Rules](https://cloud.google.com/storage/docs/lifecycle#configuration) configuration. Multiple blocks of this type are permitted. Structure is documented below.
        :param pulumi.Input[str] location: The [GCS location](https://cloud.google.com/storage/docs/bucket-locations)
        :param pulumi.Input[dict] logging: The bucket's [Access & Storage Logs](https://cloud.google.com/storage/docs/access-logs) configuration.
        :param pulumi.Input[str] name: The name of the bucket.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[bool] requester_pays: Enables [Requester Pays](https://cloud.google.com/storage/docs/requester-pays) on a storage bucket.
        :param pulumi.Input[dict] retention_policy: Configuration of the bucket's data retention policy for how long objects in the bucket should be retained. Structure is documented below.
        :param pulumi.Input[str] storage_class: The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
        :param pulumi.Input[dict] versioning: The bucket's [Versioning](https://cloud.google.com/storage/docs/object-versioning) configuration.
        :param pulumi.Input[dict] website: Configuration if the bucket acts as a website. Structure is documented below.

        The **cors** object supports the following:

          * `maxAgeSeconds` (`pulumi.Input[float]`) - The value, in seconds, to return in the [Access-Control-Max-Age header](https://www.w3.org/TR/cors/#access-control-max-age-response-header) used in preflight responses.
          * `methods` (`pulumi.Input[list]`) - The list of HTTP methods on which to include CORS response headers, (GET, OPTIONS, POST, etc) Note: "*" is permitted in the list of methods, and means "any method".
          * `origins` (`pulumi.Input[list]`) - The list of [Origins](https://tools.ietf.org/html/rfc6454) eligible to receive CORS response headers. Note: "*" is permitted in the list of origins, and means "any Origin".
          * `responseHeaders` (`pulumi.Input[list]`) - The list of HTTP headers other than the [simple response headers](https://www.w3.org/TR/cors/#simple-response-header) to give permission for the user-agent to share across domains.

        The **encryption** object supports the following:

          * `defaultKmsKeyName` (`pulumi.Input[str]`)

        The **lifecycle_rules** object supports the following:

          * `action` (`pulumi.Input[dict]`) - The Lifecycle Rule's action configuration. A single block of this type is supported. Structure is documented below.
            * `storage_class` (`pulumi.Input[str]`) - The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
            * `type` (`pulumi.Input[str]`) - The type of the action of this Lifecycle Rule. Supported values include: `Delete` and `SetStorageClass`.

          * `condition` (`pulumi.Input[dict]`) - The Lifecycle Rule's condition configuration. A single block of this type is supported. Structure is documented below.
            * `age` (`pulumi.Input[float]`) - Minimum age of an object in days to satisfy this condition.
            * `createdBefore` (`pulumi.Input[str]`) - Creation date of an object in RFC 3339 (e.g. `2017-06-13`) to satisfy this condition.
            * `matchesStorageClasses` (`pulumi.Input[list]`) - [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects to satisfy this condition. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`, `STANDARD`, `DURABLE_REDUCED_AVAILABILITY`.
            * `numNewerVersions` (`pulumi.Input[float]`) - Relevant only for versioned objects. The number of newer versions of an object to satisfy this condition.
            * `withState` (`pulumi.Input[str]`) - Match to live and/or archived objects. Unversioned buckets have only live objects. Supported values include: `"LIVE"`, `"ARCHIVED"`, `"ANY"`.

        The **logging** object supports the following:

          * `logBucket` (`pulumi.Input[str]`) - The bucket that will receive log objects.
          * `logObjectPrefix` (`pulumi.Input[str]`) - The object prefix for log objects. If it's not provided,
            by default GCS sets this to this bucket's name.

        The **retention_policy** object supports the following:

          * `isLocked` (`pulumi.Input[bool]`) - If set to `true`, the bucket will be [locked](https://cloud.google.com/storage/docs/using-bucket-lock#lock-bucket) and permanently restrict edits to the bucket's retention policy.  Caution: Locking a bucket is an irreversible action.
          * `retentionPeriod` (`pulumi.Input[float]`) - The period of time, in seconds, that objects in the bucket must be retained and cannot be deleted, overwritten, or archived. The value must be less than 3,155,760,000 seconds.

        The **versioning** object supports the following:

          * `enabled` (`pulumi.Input[bool]`) - While set to `true`, versioning is fully enabled for this bucket.

        The **website** object supports the following:

          * `mainPageSuffix` (`pulumi.Input[str]`) - Behaves as the bucket's directory index where
            missing objects are treated as potential directories.
          * `notFoundPage` (`pulumi.Input[str]`) - The custom object to return when a requested
            resource is not found.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['bucket_policy_only'] = bucket_policy_only
            __props__['cors'] = cors
            __props__['default_event_based_hold'] = default_event_based_hold
            __props__['encryption'] = encryption
            __props__['force_destroy'] = force_destroy
            __props__['labels'] = labels
            __props__['lifecycle_rules'] = lifecycle_rules
            __props__['location'] = location
            __props__['logging'] = logging
            __props__['name'] = name
            __props__['project'] = project
            __props__['requester_pays'] = requester_pays
            __props__['retention_policy'] = retention_policy
            __props__['storage_class'] = storage_class
            __props__['versioning'] = versioning
            __props__['website'] = website
            __props__['self_link'] = None
            __props__['url'] = None
        super(Bucket, __self__).__init__(
            'gcp:storage/bucket:Bucket',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bucket_policy_only=None, cors=None, default_event_based_hold=None, encryption=None, force_destroy=None, labels=None, lifecycle_rules=None, location=None, logging=None, name=None, project=None, requester_pays=None, retention_policy=None, self_link=None, storage_class=None, url=None, versioning=None, website=None):
        """
        Get an existing Bucket resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] bucket_policy_only: Enables [Bucket Policy Only](https://cloud.google.com/storage/docs/bucket-policy-only) access to a bucket.
        :param pulumi.Input[list] cors: The bucket's [Cross-Origin Resource Sharing (CORS)](https://www.w3.org/TR/cors/) configuration. Multiple blocks of this type are permitted. Structure is documented below.
        :param pulumi.Input[dict] encryption: The bucket's encryption configuration.
        :param pulumi.Input[bool] force_destroy: When deleting a bucket, this
               boolean option will delete all contained objects. If you try to delete a
               bucket that contains objects, the provider will fail that run.
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to the bucket.
        :param pulumi.Input[list] lifecycle_rules: The bucket's [Lifecycle Rules](https://cloud.google.com/storage/docs/lifecycle#configuration) configuration. Multiple blocks of this type are permitted. Structure is documented below.
        :param pulumi.Input[str] location: The [GCS location](https://cloud.google.com/storage/docs/bucket-locations)
        :param pulumi.Input[dict] logging: The bucket's [Access & Storage Logs](https://cloud.google.com/storage/docs/access-logs) configuration.
        :param pulumi.Input[str] name: The name of the bucket.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[bool] requester_pays: Enables [Requester Pays](https://cloud.google.com/storage/docs/requester-pays) on a storage bucket.
        :param pulumi.Input[dict] retention_policy: Configuration of the bucket's data retention policy for how long objects in the bucket should be retained. Structure is documented below.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        :param pulumi.Input[str] storage_class: The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
        :param pulumi.Input[str] url: The base URL of the bucket, in the format `gs://<bucket-name>`.
        :param pulumi.Input[dict] versioning: The bucket's [Versioning](https://cloud.google.com/storage/docs/object-versioning) configuration.
        :param pulumi.Input[dict] website: Configuration if the bucket acts as a website. Structure is documented below.

        The **cors** object supports the following:

          * `maxAgeSeconds` (`pulumi.Input[float]`) - The value, in seconds, to return in the [Access-Control-Max-Age header](https://www.w3.org/TR/cors/#access-control-max-age-response-header) used in preflight responses.
          * `methods` (`pulumi.Input[list]`) - The list of HTTP methods on which to include CORS response headers, (GET, OPTIONS, POST, etc) Note: "*" is permitted in the list of methods, and means "any method".
          * `origins` (`pulumi.Input[list]`) - The list of [Origins](https://tools.ietf.org/html/rfc6454) eligible to receive CORS response headers. Note: "*" is permitted in the list of origins, and means "any Origin".
          * `responseHeaders` (`pulumi.Input[list]`) - The list of HTTP headers other than the [simple response headers](https://www.w3.org/TR/cors/#simple-response-header) to give permission for the user-agent to share across domains.

        The **encryption** object supports the following:

          * `defaultKmsKeyName` (`pulumi.Input[str]`)

        The **lifecycle_rules** object supports the following:

          * `action` (`pulumi.Input[dict]`) - The Lifecycle Rule's action configuration. A single block of this type is supported. Structure is documented below.
            * `storage_class` (`pulumi.Input[str]`) - The target [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects affected by this Lifecycle Rule. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`.
            * `type` (`pulumi.Input[str]`) - The type of the action of this Lifecycle Rule. Supported values include: `Delete` and `SetStorageClass`.

          * `condition` (`pulumi.Input[dict]`) - The Lifecycle Rule's condition configuration. A single block of this type is supported. Structure is documented below.
            * `age` (`pulumi.Input[float]`) - Minimum age of an object in days to satisfy this condition.
            * `createdBefore` (`pulumi.Input[str]`) - Creation date of an object in RFC 3339 (e.g. `2017-06-13`) to satisfy this condition.
            * `matchesStorageClasses` (`pulumi.Input[list]`) - [Storage Class](https://cloud.google.com/storage/docs/storage-classes) of objects to satisfy this condition. Supported values include: `MULTI_REGIONAL`, `REGIONAL`, `NEARLINE`, `COLDLINE`, `STANDARD`, `DURABLE_REDUCED_AVAILABILITY`.
            * `numNewerVersions` (`pulumi.Input[float]`) - Relevant only for versioned objects. The number of newer versions of an object to satisfy this condition.
            * `withState` (`pulumi.Input[str]`) - Match to live and/or archived objects. Unversioned buckets have only live objects. Supported values include: `"LIVE"`, `"ARCHIVED"`, `"ANY"`.

        The **logging** object supports the following:

          * `logBucket` (`pulumi.Input[str]`) - The bucket that will receive log objects.
          * `logObjectPrefix` (`pulumi.Input[str]`) - The object prefix for log objects. If it's not provided,
            by default GCS sets this to this bucket's name.

        The **retention_policy** object supports the following:

          * `isLocked` (`pulumi.Input[bool]`) - If set to `true`, the bucket will be [locked](https://cloud.google.com/storage/docs/using-bucket-lock#lock-bucket) and permanently restrict edits to the bucket's retention policy.  Caution: Locking a bucket is an irreversible action.
          * `retentionPeriod` (`pulumi.Input[float]`) - The period of time, in seconds, that objects in the bucket must be retained and cannot be deleted, overwritten, or archived. The value must be less than 3,155,760,000 seconds.

        The **versioning** object supports the following:

          * `enabled` (`pulumi.Input[bool]`) - While set to `true`, versioning is fully enabled for this bucket.

        The **website** object supports the following:

          * `mainPageSuffix` (`pulumi.Input[str]`) - Behaves as the bucket's directory index where
            missing objects are treated as potential directories.
          * `notFoundPage` (`pulumi.Input[str]`) - The custom object to return when a requested
            resource is not found.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bucket_policy_only"] = bucket_policy_only
        __props__["cors"] = cors
        __props__["default_event_based_hold"] = default_event_based_hold
        __props__["encryption"] = encryption
        __props__["force_destroy"] = force_destroy
        __props__["labels"] = labels
        __props__["lifecycle_rules"] = lifecycle_rules
        __props__["location"] = location
        __props__["logging"] = logging
        __props__["name"] = name
        __props__["project"] = project
        __props__["requester_pays"] = requester_pays
        __props__["retention_policy"] = retention_policy
        __props__["self_link"] = self_link
        __props__["storage_class"] = storage_class
        __props__["url"] = url
        __props__["versioning"] = versioning
        __props__["website"] = website
        return Bucket(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class ResourcePolicy(pulumi.CustomResource):
    group_placement_policy: pulumi.Output[dict]
    """
    Policy for creating snapshots of persistent disks.  Structure is documented below.

      * `availabilityDomainCount` (`float`) - The number of availability domains instances will be spread across. If two instances are in different
        availability domain, they will not be put in the same low latency network
      * `collocation` (`str`) - Collocation specifies whether to place VMs inside the same availability domain on the same low-latency network.
        Specify `COLLOCATED` to enable collocation. Can only be specified with `vm_count`. If compute instances are created
        with a COLLOCATED policy, then exactly `vm_count` instances must be created at the same time with the resource policy
        attached.
      * `vmCount` (`float`) - Number of vms in this placement group.
    """
    name: pulumi.Output[str]
    """
    The name of the resource, provided by the client when initially creating
    the resource. The resource name must be 1-63 characters long, and comply
    with RFC1035. Specifically, the name must be 1-63 characters long and
    match the regular expression `a-z`? which means the
    first character must be a lowercase letter, and all following characters
    must be a dash, lowercase letter, or digit, except the last character,
    which cannot be a dash.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    Region where resource policy resides.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    snapshot_schedule_policy: pulumi.Output[dict]
    """
    Policy for creating snapshots of persistent disks.  Structure is documented below.

      * `retention_policy` (`dict`) - Retention policy applied to snapshots created by this resource policy.  Structure is documented below.
        * `maxRetentionDays` (`float`) - Maximum age of the snapshot that is allowed to be kept.
        * `onSourceDiskDelete` (`str`) - Specifies the behavior to apply to scheduled snapshots when
          the source disk is deleted.

      * `schedule` (`dict`) - Contains one of an `hourlySchedule`, `dailySchedule`, or `weeklySchedule`.  Structure is documented below.
        * `dailySchedule` (`dict`) - The policy will execute every nth day at the specified time.  Structure is documented below.
          * `daysInCycle` (`float`) - The number of days between snapshots.
          * `startTime` (`str`) - Time within the window to start the operations.
            It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

        * `hourlySchedule` (`dict`) - The policy will execute every nth hour starting at the specified time.  Structure is documented below.
          * `hoursInCycle` (`float`) - The number of hours between snapshots.
          * `startTime` (`str`) - Time within the window to start the operations.
            It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

        * `weeklySchedule` (`dict`) - Allows specifying a snapshot time for each day of the week.  Structure is documented below.
          * `dayOfWeeks` (`list`) - May contain up to seven (one for each day of the week) snapshot times.  Structure is documented below.
            * `day` (`str`) - The day of the week to create the snapshot. e.g. MONDAY
            * `startTime` (`str`) - Time within the window to start the operations.
              It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

      * `snapshotProperties` (`dict`) - Properties with which the snapshots are created, such as labels.  Structure is documented below.
        * `guestFlush` (`bool`) - Whether to perform a 'guest aware' snapshot.
        * `labels` (`dict`) - A set of key-value pairs.
        * `storageLocations` (`str`) - Cloud Storage bucket location to store the auto snapshot
          (regional or multi-regional)
    """
    def __init__(__self__, resource_name, opts=None, group_placement_policy=None, name=None, project=None, region=None, snapshot_schedule_policy=None, __props__=None, __name__=None, __opts__=None):
        """
        A policy that can be attached to a resource to specify or schedule actions on that resource.



        ## Example Usage

        ### Resource Policy Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        foo = gcp.compute.ResourcePolicy("foo",
            region="us-central1",
            snapshot_schedule_policy={
                "schedule": {
                    "dailySchedule": {
                        "daysInCycle": 1,
                        "startTime": "04:00",
                    },
                },
            })
        ```

        ### Resource Policy Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bar = gcp.compute.ResourcePolicy("bar",
            region="us-central1",
            snapshot_schedule_policy={
                "retention_policy": {
                    "maxRetentionDays": 10,
                    "onSourceDiskDelete": "KEEP_AUTO_SNAPSHOTS",
                },
                "schedule": {
                    "hourlySchedule": {
                        "hoursInCycle": 20,
                        "startTime": "23:00",
                    },
                },
                "snapshotProperties": {
                    "guestFlush": True,
                    "labels": {
                        "myLabel": "value",
                    },
                    "storageLocations": "us",
                },
            })
        ```

        ### Resource Policy Placement Policy

        ```python
        import pulumi
        import pulumi_gcp as gcp

        baz = gcp.compute.ResourcePolicy("baz",
            group_placement_policy={
                "collocation": "COLLOCATED",
                "vmCount": 2,
            },
            region="us-central1")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] group_placement_policy: Policy for creating snapshots of persistent disks.  Structure is documented below.
        :param pulumi.Input[str] name: The name of the resource, provided by the client when initially creating
               the resource. The resource name must be 1-63 characters long, and comply
               with RFC1035. Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z`? which means the
               first character must be a lowercase letter, and all following characters
               must be a dash, lowercase letter, or digit, except the last character,
               which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where resource policy resides.
        :param pulumi.Input[dict] snapshot_schedule_policy: Policy for creating snapshots of persistent disks.  Structure is documented below.

        The **group_placement_policy** object supports the following:

          * `availabilityDomainCount` (`pulumi.Input[float]`) - The number of availability domains instances will be spread across. If two instances are in different
            availability domain, they will not be put in the same low latency network
          * `collocation` (`pulumi.Input[str]`) - Collocation specifies whether to place VMs inside the same availability domain on the same low-latency network.
            Specify `COLLOCATED` to enable collocation. Can only be specified with `vm_count`. If compute instances are created
            with a COLLOCATED policy, then exactly `vm_count` instances must be created at the same time with the resource policy
            attached.
          * `vmCount` (`pulumi.Input[float]`) - Number of vms in this placement group.

        The **snapshot_schedule_policy** object supports the following:

          * `retention_policy` (`pulumi.Input[dict]`) - Retention policy applied to snapshots created by this resource policy.  Structure is documented below.
            * `maxRetentionDays` (`pulumi.Input[float]`) - Maximum age of the snapshot that is allowed to be kept.
            * `onSourceDiskDelete` (`pulumi.Input[str]`) - Specifies the behavior to apply to scheduled snapshots when
              the source disk is deleted.

          * `schedule` (`pulumi.Input[dict]`) - Contains one of an `hourlySchedule`, `dailySchedule`, or `weeklySchedule`.  Structure is documented below.
            * `dailySchedule` (`pulumi.Input[dict]`) - The policy will execute every nth day at the specified time.  Structure is documented below.
              * `daysInCycle` (`pulumi.Input[float]`) - The number of days between snapshots.
              * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

            * `hourlySchedule` (`pulumi.Input[dict]`) - The policy will execute every nth hour starting at the specified time.  Structure is documented below.
              * `hoursInCycle` (`pulumi.Input[float]`) - The number of hours between snapshots.
              * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

            * `weeklySchedule` (`pulumi.Input[dict]`) - Allows specifying a snapshot time for each day of the week.  Structure is documented below.
              * `dayOfWeeks` (`pulumi.Input[list]`) - May contain up to seven (one for each day of the week) snapshot times.  Structure is documented below.
                * `day` (`pulumi.Input[str]`) - The day of the week to create the snapshot. e.g. MONDAY
                * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                  It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

          * `snapshotProperties` (`pulumi.Input[dict]`) - Properties with which the snapshots are created, such as labels.  Structure is documented below.
            * `guestFlush` (`pulumi.Input[bool]`) - Whether to perform a 'guest aware' snapshot.
            * `labels` (`pulumi.Input[dict]`) - A set of key-value pairs.
            * `storageLocations` (`pulumi.Input[str]`) - Cloud Storage bucket location to store the auto snapshot
              (regional or multi-regional)
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['group_placement_policy'] = group_placement_policy
            __props__['name'] = name
            __props__['project'] = project
            __props__['region'] = region
            __props__['snapshot_schedule_policy'] = snapshot_schedule_policy
            __props__['self_link'] = None
        super(ResourcePolicy, __self__).__init__(
            'gcp:compute/resourcePolicy:ResourcePolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, group_placement_policy=None, name=None, project=None, region=None, self_link=None, snapshot_schedule_policy=None):
        """
        Get an existing ResourcePolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] group_placement_policy: Policy for creating snapshots of persistent disks.  Structure is documented below.
        :param pulumi.Input[str] name: The name of the resource, provided by the client when initially creating
               the resource. The resource name must be 1-63 characters long, and comply
               with RFC1035. Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z`? which means the
               first character must be a lowercase letter, and all following characters
               must be a dash, lowercase letter, or digit, except the last character,
               which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where resource policy resides.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        :param pulumi.Input[dict] snapshot_schedule_policy: Policy for creating snapshots of persistent disks.  Structure is documented below.

        The **group_placement_policy** object supports the following:

          * `availabilityDomainCount` (`pulumi.Input[float]`) - The number of availability domains instances will be spread across. If two instances are in different
            availability domain, they will not be put in the same low latency network
          * `collocation` (`pulumi.Input[str]`) - Collocation specifies whether to place VMs inside the same availability domain on the same low-latency network.
            Specify `COLLOCATED` to enable collocation. Can only be specified with `vm_count`. If compute instances are created
            with a COLLOCATED policy, then exactly `vm_count` instances must be created at the same time with the resource policy
            attached.
          * `vmCount` (`pulumi.Input[float]`) - Number of vms in this placement group.

        The **snapshot_schedule_policy** object supports the following:

          * `retention_policy` (`pulumi.Input[dict]`) - Retention policy applied to snapshots created by this resource policy.  Structure is documented below.
            * `maxRetentionDays` (`pulumi.Input[float]`) - Maximum age of the snapshot that is allowed to be kept.
            * `onSourceDiskDelete` (`pulumi.Input[str]`) - Specifies the behavior to apply to scheduled snapshots when
              the source disk is deleted.

          * `schedule` (`pulumi.Input[dict]`) - Contains one of an `hourlySchedule`, `dailySchedule`, or `weeklySchedule`.  Structure is documented below.
            * `dailySchedule` (`pulumi.Input[dict]`) - The policy will execute every nth day at the specified time.  Structure is documented below.
              * `daysInCycle` (`pulumi.Input[float]`) - The number of days between snapshots.
              * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

            * `hourlySchedule` (`pulumi.Input[dict]`) - The policy will execute every nth hour starting at the specified time.  Structure is documented below.
              * `hoursInCycle` (`pulumi.Input[float]`) - The number of hours between snapshots.
              * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

            * `weeklySchedule` (`pulumi.Input[dict]`) - Allows specifying a snapshot time for each day of the week.  Structure is documented below.
              * `dayOfWeeks` (`pulumi.Input[list]`) - May contain up to seven (one for each day of the week) snapshot times.  Structure is documented below.
                * `day` (`pulumi.Input[str]`) - The day of the week to create the snapshot. e.g. MONDAY
                * `startTime` (`pulumi.Input[str]`) - Time within the window to start the operations.
                  It must be in format "HH:MM", where HH : [00-23] and MM : [00-00] GMT.

          * `snapshotProperties` (`pulumi.Input[dict]`) - Properties with which the snapshots are created, such as labels.  Structure is documented below.
            * `guestFlush` (`pulumi.Input[bool]`) - Whether to perform a 'guest aware' snapshot.
            * `labels` (`pulumi.Input[dict]`) - A set of key-value pairs.
            * `storageLocations` (`pulumi.Input[str]`) - Cloud Storage bucket location to store the auto snapshot
              (regional or multi-regional)
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["group_placement_policy"] = group_placement_policy
        __props__["name"] = name
        __props__["project"] = project
        __props__["region"] = region
        __props__["self_link"] = self_link
        __props__["snapshot_schedule_policy"] = snapshot_schedule_policy
        return ResourcePolicy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

